# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Any, Dict, Union

from datadog_api_client.api_client import ApiClient, Endpoint as _Endpoint
from datadog_api_client.model_utils import (
    UnsetType,
    unset,
)
from datadog_api_client.v2.model.metrics_and_metric_tag_configurations_response import (
    MetricsAndMetricTagConfigurationsResponse,
)
from datadog_api_client.v2.model.metric_tag_configuration_metric_types import MetricTagConfigurationMetricTypes
from datadog_api_client.v2.model.metric_bulk_tag_config_response import MetricBulkTagConfigResponse
from datadog_api_client.v2.model.metric_bulk_tag_config_delete_request import MetricBulkTagConfigDeleteRequest
from datadog_api_client.v2.model.metric_bulk_tag_config_create_request import MetricBulkTagConfigCreateRequest
from datadog_api_client.v2.model.metric_suggested_tags_and_aggregations_response import (
    MetricSuggestedTagsAndAggregationsResponse,
)
from datadog_api_client.v2.model.metric_all_tags_response import MetricAllTagsResponse
from datadog_api_client.v2.model.metric_estimate_response import MetricEstimateResponse
from datadog_api_client.v2.model.metric_tag_configuration_response import MetricTagConfigurationResponse
from datadog_api_client.v2.model.metric_tag_configuration_update_request import MetricTagConfigurationUpdateRequest
from datadog_api_client.v2.model.metric_tag_configuration_create_request import MetricTagConfigurationCreateRequest
from datadog_api_client.v2.model.metric_volumes_response import MetricVolumesResponse
from datadog_api_client.v2.model.scalar_formula_query_response import ScalarFormulaQueryResponse
from datadog_api_client.v2.model.scalar_formula_query_request import ScalarFormulaQueryRequest
from datadog_api_client.v2.model.timeseries_formula_query_response import TimeseriesFormulaQueryResponse
from datadog_api_client.v2.model.timeseries_formula_query_request import TimeseriesFormulaQueryRequest
from datadog_api_client.v2.model.intake_payload_accepted import IntakePayloadAccepted
from datadog_api_client.v2.model.metric_content_encoding import MetricContentEncoding
from datadog_api_client.v2.model.metric_payload import MetricPayload


class MetricsApi:
    """
    The metrics endpoint allows you to:

    * Post metrics data so it can be graphed on Datadog’s dashboards
    * Query metrics from any time period (timeseries and scalar)
    * Modify tag configurations for metrics
    * View tags and volumes for metrics

    **Note** : A graph can only contain a set number of points
    and as the timeframe over which a metric is viewed increases,
    aggregation between points occurs to stay below that set number.

    The Post, Patch, and Delete ``manage_tags`` API methods can only be performed by
    a user who has the ``Manage Tags for Metrics`` permission.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        self._create_bulk_tags_metrics_configuration_endpoint = _Endpoint(
            settings={
                "response_type": (MetricBulkTagConfigResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth"],
                "endpoint_path": "/api/v2/metrics/config/bulk-tags",
                "operation_id": "create_bulk_tags_metrics_configuration",
                "http_method": "POST",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "body": {
                    "required": True,
                    "openapi_types": (MetricBulkTagConfigCreateRequest,),
                    "location": "body",
                },
            },
            headers_map={"accept": ["application/json"], "content_type": ["application/json"]},
            api_client=api_client,
        )

        self._create_tag_configuration_endpoint = _Endpoint(
            settings={
                "response_type": (MetricTagConfigurationResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth"],
                "endpoint_path": "/api/v2/metrics/{metric_name}/tags",
                "operation_id": "create_tag_configuration",
                "http_method": "POST",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "metric_name": {
                    "required": True,
                    "openapi_types": (str,),
                    "attribute": "metric_name",
                    "location": "path",
                },
                "body": {
                    "required": True,
                    "openapi_types": (MetricTagConfigurationCreateRequest,),
                    "location": "body",
                },
            },
            headers_map={"accept": ["application/json"], "content_type": ["application/json"]},
            api_client=api_client,
        )

        self._delete_bulk_tags_metrics_configuration_endpoint = _Endpoint(
            settings={
                "response_type": (MetricBulkTagConfigResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth"],
                "endpoint_path": "/api/v2/metrics/config/bulk-tags",
                "operation_id": "delete_bulk_tags_metrics_configuration",
                "http_method": "DELETE",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "body": {
                    "required": True,
                    "openapi_types": (MetricBulkTagConfigDeleteRequest,),
                    "location": "body",
                },
            },
            headers_map={"accept": ["application/json"], "content_type": ["application/json"]},
            api_client=api_client,
        )

        self._delete_tag_configuration_endpoint = _Endpoint(
            settings={
                "response_type": None,
                "auth": ["apiKeyAuth", "appKeyAuth"],
                "endpoint_path": "/api/v2/metrics/{metric_name}/tags",
                "operation_id": "delete_tag_configuration",
                "http_method": "DELETE",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "metric_name": {
                    "required": True,
                    "openapi_types": (str,),
                    "attribute": "metric_name",
                    "location": "path",
                },
            },
            headers_map={
                "accept": ["*/*"],
                "content_type": [],
            },
            api_client=api_client,
        )

        self._estimate_metrics_output_series_endpoint = _Endpoint(
            settings={
                "response_type": (MetricEstimateResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth", "AuthZ"],
                "endpoint_path": "/api/v2/metrics/{metric_name}/estimate",
                "operation_id": "estimate_metrics_output_series",
                "http_method": "GET",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "metric_name": {
                    "required": True,
                    "openapi_types": (str,),
                    "attribute": "metric_name",
                    "location": "path",
                },
                "filter_groups": {
                    "openapi_types": (str,),
                    "attribute": "filter[groups]",
                    "location": "query",
                },
                "filter_hours_ago": {
                    "validation": {
                        "inclusive_maximum": 2147483647,
                    },
                    "openapi_types": (int,),
                    "attribute": "filter[hours_ago]",
                    "location": "query",
                },
                "filter_num_aggregations": {
                    "validation": {
                        "inclusive_maximum": 9,
                    },
                    "openapi_types": (int,),
                    "attribute": "filter[num_aggregations]",
                    "location": "query",
                },
                "filter_pct": {
                    "openapi_types": (bool,),
                    "attribute": "filter[pct]",
                    "location": "query",
                },
                "filter_timespan_h": {
                    "validation": {
                        "inclusive_maximum": 2147483647,
                    },
                    "openapi_types": (int,),
                    "attribute": "filter[timespan_h]",
                    "location": "query",
                },
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )

        self._list_active_metric_configurations_endpoint = _Endpoint(
            settings={
                "response_type": (MetricSuggestedTagsAndAggregationsResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth", "AuthZ"],
                "endpoint_path": "/api/v2/metrics/{metric_name}/active-configurations",
                "operation_id": "list_active_metric_configurations",
                "http_method": "GET",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "metric_name": {
                    "required": True,
                    "openapi_types": (str,),
                    "attribute": "metric_name",
                    "location": "path",
                },
                "window_seconds": {
                    "openapi_types": (int,),
                    "attribute": "window[seconds]",
                    "location": "query",
                },
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )

        self._list_tag_configuration_by_name_endpoint = _Endpoint(
            settings={
                "response_type": (MetricTagConfigurationResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth", "AuthZ"],
                "endpoint_path": "/api/v2/metrics/{metric_name}/tags",
                "operation_id": "list_tag_configuration_by_name",
                "http_method": "GET",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "metric_name": {
                    "required": True,
                    "openapi_types": (str,),
                    "attribute": "metric_name",
                    "location": "path",
                },
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )

        self._list_tag_configurations_endpoint = _Endpoint(
            settings={
                "response_type": (MetricsAndMetricTagConfigurationsResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth", "AuthZ"],
                "endpoint_path": "/api/v2/metrics",
                "operation_id": "list_tag_configurations",
                "http_method": "GET",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "filter_configured": {
                    "openapi_types": (bool,),
                    "attribute": "filter[configured]",
                    "location": "query",
                },
                "filter_tags_configured": {
                    "openapi_types": (str,),
                    "attribute": "filter[tags_configured]",
                    "location": "query",
                },
                "filter_metric_type": {
                    "openapi_types": (MetricTagConfigurationMetricTypes,),
                    "attribute": "filter[metric_type]",
                    "location": "query",
                },
                "filter_include_percentiles": {
                    "openapi_types": (bool,),
                    "attribute": "filter[include_percentiles]",
                    "location": "query",
                },
                "filter_queried": {
                    "openapi_types": (bool,),
                    "attribute": "filter[queried]",
                    "location": "query",
                },
                "filter_tags": {
                    "openapi_types": (str,),
                    "attribute": "filter[tags]",
                    "location": "query",
                },
                "window_seconds": {
                    "openapi_types": (int,),
                    "attribute": "window[seconds]",
                    "location": "query",
                },
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )

        self._list_tags_by_metric_name_endpoint = _Endpoint(
            settings={
                "response_type": (MetricAllTagsResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth", "AuthZ"],
                "endpoint_path": "/api/v2/metrics/{metric_name}/all-tags",
                "operation_id": "list_tags_by_metric_name",
                "http_method": "GET",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "metric_name": {
                    "required": True,
                    "openapi_types": (str,),
                    "attribute": "metric_name",
                    "location": "path",
                },
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )

        self._list_volumes_by_metric_name_endpoint = _Endpoint(
            settings={
                "response_type": (MetricVolumesResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth", "AuthZ"],
                "endpoint_path": "/api/v2/metrics/{metric_name}/volumes",
                "operation_id": "list_volumes_by_metric_name",
                "http_method": "GET",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "metric_name": {
                    "required": True,
                    "openapi_types": (str,),
                    "attribute": "metric_name",
                    "location": "path",
                },
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )

        self._query_scalar_data_endpoint = _Endpoint(
            settings={
                "response_type": (ScalarFormulaQueryResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth"],
                "endpoint_path": "/api/v2/query/scalar",
                "operation_id": "query_scalar_data",
                "http_method": "POST",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "body": {
                    "required": True,
                    "openapi_types": (ScalarFormulaQueryRequest,),
                    "location": "body",
                },
            },
            headers_map={"accept": ["application/json"], "content_type": ["application/json"]},
            api_client=api_client,
        )

        self._query_timeseries_data_endpoint = _Endpoint(
            settings={
                "response_type": (TimeseriesFormulaQueryResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth"],
                "endpoint_path": "/api/v2/query/timeseries",
                "operation_id": "query_timeseries_data",
                "http_method": "POST",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "body": {
                    "required": True,
                    "openapi_types": (TimeseriesFormulaQueryRequest,),
                    "location": "body",
                },
            },
            headers_map={"accept": ["application/json"], "content_type": ["application/json"]},
            api_client=api_client,
        )

        self._submit_metrics_endpoint = _Endpoint(
            settings={
                "response_type": (IntakePayloadAccepted,),
                "auth": ["apiKeyAuth"],
                "endpoint_path": "/api/v2/series",
                "operation_id": "submit_metrics",
                "http_method": "POST",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "content_encoding": {
                    "openapi_types": (MetricContentEncoding,),
                    "attribute": "Content-Encoding",
                    "location": "header",
                },
                "body": {
                    "required": True,
                    "openapi_types": (MetricPayload,),
                    "location": "body",
                },
            },
            headers_map={"accept": ["application/json"], "content_type": ["application/json"]},
            api_client=api_client,
        )

        self._update_tag_configuration_endpoint = _Endpoint(
            settings={
                "response_type": (MetricTagConfigurationResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth"],
                "endpoint_path": "/api/v2/metrics/{metric_name}/tags",
                "operation_id": "update_tag_configuration",
                "http_method": "PATCH",
                "version": "v2",
                "servers": None,
            },
            params_map={
                "metric_name": {
                    "required": True,
                    "openapi_types": (str,),
                    "attribute": "metric_name",
                    "location": "path",
                },
                "body": {
                    "required": True,
                    "openapi_types": (MetricTagConfigurationUpdateRequest,),
                    "location": "body",
                },
            },
            headers_map={"accept": ["application/json"], "content_type": ["application/json"]},
            api_client=api_client,
        )

    def create_bulk_tags_metrics_configuration(
        self,
        body: MetricBulkTagConfigCreateRequest,
    ) -> MetricBulkTagConfigResponse:
        """Configure tags for multiple metrics.

        Create and define a list of queryable tag keys for a set of existing count, gauge, rate, and distribution metrics.
        Metrics are selected by passing a metric name prefix. Use the Delete method of this API path to remove tag configurations.
        Results can be sent to a set of account email addresses, just like the same operation in the Datadog web app.
        If multiple calls include the same metric, the last configuration applied (not by submit order) is used, do not
        expect deterministic ordering of concurrent calls.
        Can only be used with application keys of users with the ``Manage Tags for Metrics`` permission.

        :type body: MetricBulkTagConfigCreateRequest
        :rtype: MetricBulkTagConfigResponse
        """
        kwargs: Dict[str, Any] = {}
        kwargs["body"] = body

        return self._create_bulk_tags_metrics_configuration_endpoint.call_with_http_info(**kwargs)

    def create_tag_configuration(
        self,
        metric_name: str,
        body: MetricTagConfigurationCreateRequest,
    ) -> MetricTagConfigurationResponse:
        """Create a tag configuration.

        Create and define a list of queryable tag keys for an existing count/gauge/rate/distribution metric.
        Optionally, include percentile aggregations on any distribution metric or configure custom aggregations
        on any count, rate, or gauge metric.
        Can only be used with application keys of users with the ``Manage Tags for Metrics`` permission.

        :param metric_name: The name of the metric.
        :type metric_name: str
        :type body: MetricTagConfigurationCreateRequest
        :rtype: MetricTagConfigurationResponse
        """
        kwargs: Dict[str, Any] = {}
        kwargs["metric_name"] = metric_name

        kwargs["body"] = body

        return self._create_tag_configuration_endpoint.call_with_http_info(**kwargs)

    def delete_bulk_tags_metrics_configuration(
        self,
        body: MetricBulkTagConfigDeleteRequest,
    ) -> MetricBulkTagConfigResponse:
        """Configure tags for multiple metrics.

        Delete all custom lists of queryable tag keys for a set of existing count, gauge, rate, and distribution metrics.
        Metrics are selected by passing a metric name prefix.
        Results can be sent to a set of account email addresses, just like the same operation in the Datadog web app.
        Can only be used with application keys of users with the ``Manage Tags for Metrics`` permission.

        :type body: MetricBulkTagConfigDeleteRequest
        :rtype: MetricBulkTagConfigResponse
        """
        kwargs: Dict[str, Any] = {}
        kwargs["body"] = body

        return self._delete_bulk_tags_metrics_configuration_endpoint.call_with_http_info(**kwargs)

    def delete_tag_configuration(
        self,
        metric_name: str,
    ) -> None:
        """Delete a tag configuration.

        Deletes a metric's tag configuration. Can only be used with application
        keys from users with the ``Manage Tags for Metrics`` permission.

        :param metric_name: The name of the metric.
        :type metric_name: str
        :rtype: None
        """
        kwargs: Dict[str, Any] = {}
        kwargs["metric_name"] = metric_name

        return self._delete_tag_configuration_endpoint.call_with_http_info(**kwargs)

    def estimate_metrics_output_series(
        self,
        metric_name: str,
        *,
        filter_groups: Union[str, UnsetType] = unset,
        filter_hours_ago: Union[int, UnsetType] = unset,
        filter_num_aggregations: Union[int, UnsetType] = unset,
        filter_pct: Union[bool, UnsetType] = unset,
        filter_timespan_h: Union[int, UnsetType] = unset,
    ) -> MetricEstimateResponse:
        """Tag Configuration Cardinality Estimator.

        Returns the estimated cardinality for a metric with a given tag, percentile and number of aggregations configuration using Metrics without Limits&trade;.

        :param metric_name: The name of the metric.
        :type metric_name: str
        :param filter_groups: Filtered tag keys that the metric is configured to query with.
        :type filter_groups: str, optional
        :param filter_hours_ago: The number of hours of look back (from now) to estimate cardinality with.
        :type filter_hours_ago: int, optional
        :param filter_num_aggregations: The number of aggregations that a ``count`` , ``rate`` , or ``gauge`` metric is configured to use. Max number of aggregation combos is 9.
        :type filter_num_aggregations: int, optional
        :param filter_pct: A boolean, for distribution metrics only, to estimate cardinality if the metric includes additional percentile aggregators.
        :type filter_pct: bool, optional
        :param filter_timespan_h: A window, in hours, from the look back to estimate cardinality with.
        :type filter_timespan_h: int, optional
        :rtype: MetricEstimateResponse
        """
        kwargs: Dict[str, Any] = {}
        kwargs["metric_name"] = metric_name

        if filter_groups is not unset:
            kwargs["filter_groups"] = filter_groups

        if filter_hours_ago is not unset:
            kwargs["filter_hours_ago"] = filter_hours_ago

        if filter_num_aggregations is not unset:
            kwargs["filter_num_aggregations"] = filter_num_aggregations

        if filter_pct is not unset:
            kwargs["filter_pct"] = filter_pct

        if filter_timespan_h is not unset:
            kwargs["filter_timespan_h"] = filter_timespan_h

        return self._estimate_metrics_output_series_endpoint.call_with_http_info(**kwargs)

    def list_active_metric_configurations(
        self,
        metric_name: str,
        *,
        window_seconds: Union[int, UnsetType] = unset,
    ) -> MetricSuggestedTagsAndAggregationsResponse:
        """List active tags and aggregations.

        List tags and aggregations that are actively queried on dashboards and monitors for a given metric name.

        :param metric_name: The name of the metric.
        :type metric_name: str
        :param window_seconds: The number of seconds of look back (from now).
            Default value is 604,800 (1 week), minimum value is 7200 (2 hours), maximum value is 2,630,000 (1 month).
        :type window_seconds: int, optional
        :rtype: MetricSuggestedTagsAndAggregationsResponse
        """
        kwargs: Dict[str, Any] = {}
        kwargs["metric_name"] = metric_name

        if window_seconds is not unset:
            kwargs["window_seconds"] = window_seconds

        return self._list_active_metric_configurations_endpoint.call_with_http_info(**kwargs)

    def list_tag_configuration_by_name(
        self,
        metric_name: str,
    ) -> MetricTagConfigurationResponse:
        """List tag configuration by name.

        Returns the tag configuration for the given metric name.

        :param metric_name: The name of the metric.
        :type metric_name: str
        :rtype: MetricTagConfigurationResponse
        """
        kwargs: Dict[str, Any] = {}
        kwargs["metric_name"] = metric_name

        return self._list_tag_configuration_by_name_endpoint.call_with_http_info(**kwargs)

    def list_tag_configurations(
        self,
        *,
        filter_configured: Union[bool, UnsetType] = unset,
        filter_tags_configured: Union[str, UnsetType] = unset,
        filter_metric_type: Union[MetricTagConfigurationMetricTypes, UnsetType] = unset,
        filter_include_percentiles: Union[bool, UnsetType] = unset,
        filter_queried: Union[bool, UnsetType] = unset,
        filter_tags: Union[str, UnsetType] = unset,
        window_seconds: Union[int, UnsetType] = unset,
    ) -> MetricsAndMetricTagConfigurationsResponse:
        """Get a list of metrics.

        Returns all metrics that can be configured in the Metrics Summary page or with Metrics without Limits™ (matching additional filters if specified).

        :param filter_configured: Filter custom metrics that have configured tags.
        :type filter_configured: bool, optional
        :param filter_tags_configured: Filter tag configurations by configured tags.
        :type filter_tags_configured: str, optional
        :param filter_metric_type: Filter metrics by metric type.
        :type filter_metric_type: MetricTagConfigurationMetricTypes, optional
        :param filter_include_percentiles: Filter distributions with additional percentile
            aggregations enabled or disabled.
        :type filter_include_percentiles: bool, optional
        :param filter_queried: Filter custom metrics that have or have not been queried in the specified window[seconds].
            If no window is provided or the window is less than 2 hours, a default of 2 hours will be applied.
        :type filter_queried: bool, optional
        :param filter_tags: Filter metrics that have been submitted with the given tags. Supports boolean and wildcard expressions.
            Can only be combined with the filter[queried] filter.
        :type filter_tags: str, optional
        :param window_seconds: The number of seconds of look back (from now) to apply to a filter[tag] or filter[queried] query.
            Defaults value is 3600 (1 hour), maximum value is 1,209,600 (2 weeks).
        :type window_seconds: int, optional
        :rtype: MetricsAndMetricTagConfigurationsResponse
        """
        kwargs: Dict[str, Any] = {}
        if filter_configured is not unset:
            kwargs["filter_configured"] = filter_configured

        if filter_tags_configured is not unset:
            kwargs["filter_tags_configured"] = filter_tags_configured

        if filter_metric_type is not unset:
            kwargs["filter_metric_type"] = filter_metric_type

        if filter_include_percentiles is not unset:
            kwargs["filter_include_percentiles"] = filter_include_percentiles

        if filter_queried is not unset:
            kwargs["filter_queried"] = filter_queried

        if filter_tags is not unset:
            kwargs["filter_tags"] = filter_tags

        if window_seconds is not unset:
            kwargs["window_seconds"] = window_seconds

        return self._list_tag_configurations_endpoint.call_with_http_info(**kwargs)

    def list_tags_by_metric_name(
        self,
        metric_name: str,
    ) -> MetricAllTagsResponse:
        """List tags by metric name.

        View indexed tag key-value pairs for a given metric name.

        :param metric_name: The name of the metric.
        :type metric_name: str
        :rtype: MetricAllTagsResponse
        """
        kwargs: Dict[str, Any] = {}
        kwargs["metric_name"] = metric_name

        return self._list_tags_by_metric_name_endpoint.call_with_http_info(**kwargs)

    def list_volumes_by_metric_name(
        self,
        metric_name: str,
    ) -> MetricVolumesResponse:
        """List distinct metric volumes by metric name.

        View distinct metrics volumes for the given metric name.

        Custom metrics generated in-app from other products will return ``null`` for ingested volumes.

        :param metric_name: The name of the metric.
        :type metric_name: str
        :rtype: MetricVolumesResponse
        """
        kwargs: Dict[str, Any] = {}
        kwargs["metric_name"] = metric_name

        return self._list_volumes_by_metric_name_endpoint.call_with_http_info(**kwargs)

    def query_scalar_data(
        self,
        body: ScalarFormulaQueryRequest,
    ) -> ScalarFormulaQueryResponse:
        """Scalar cross product query.

        The internal endpoint to query scalar/table data for multiple data sources and
        process the data using formulas and functions.

        :type body: ScalarFormulaQueryRequest
        :rtype: ScalarFormulaQueryResponse
        """
        kwargs: Dict[str, Any] = {}
        kwargs["body"] = body

        return self._query_scalar_data_endpoint.call_with_http_info(**kwargs)

    def query_timeseries_data(
        self,
        body: TimeseriesFormulaQueryRequest,
    ) -> TimeseriesFormulaQueryResponse:
        """Timeseries cross product query.

        The internal endpoint to query timeseries data for multiple data sources and
        process the data by applying formulas and functions.

        :type body: TimeseriesFormulaQueryRequest
        :rtype: TimeseriesFormulaQueryResponse
        """
        kwargs: Dict[str, Any] = {}
        kwargs["body"] = body

        return self._query_timeseries_data_endpoint.call_with_http_info(**kwargs)

    def submit_metrics(
        self,
        body: MetricPayload,
        *,
        content_encoding: Union[MetricContentEncoding, UnsetType] = unset,
    ) -> IntakePayloadAccepted:
        """Submit metrics.

        The metrics end-point allows you to post time-series data that can be graphed on Datadog’s dashboards.
        The maximum payload size is 500 kilobytes (512000 bytes). Compressed payloads must have a decompressed size of less than 5 megabytes (5242880 bytes).

        If you’re submitting metrics directly to the Datadog API without using DogStatsD, expect:

        * 64 bits for the timestamp
        * 64 bits for the value
        * 20 bytes for the metric names
        * 50 bytes for the timeseries
        * The full payload is approximately 100 bytes.

        Host name is one of the resources in the Resources field.

        :type body: MetricPayload
        :param content_encoding: HTTP header used to compress the media-type.
        :type content_encoding: MetricContentEncoding, optional
        :rtype: IntakePayloadAccepted
        """
        kwargs: Dict[str, Any] = {}
        if content_encoding is not unset:
            kwargs["content_encoding"] = content_encoding

        kwargs["body"] = body

        return self._submit_metrics_endpoint.call_with_http_info(**kwargs)

    def update_tag_configuration(
        self,
        metric_name: str,
        body: MetricTagConfigurationUpdateRequest,
    ) -> MetricTagConfigurationResponse:
        """Update a tag configuration.

        Update the tag configuration of a metric or percentile aggregations of a distribution metric or custom aggregations
        of a count, rate, or gauge metric.
        Can only be used with application keys from users with the ``Manage Tags for Metrics`` permission.

        :param metric_name: The name of the metric.
        :type metric_name: str
        :type body: MetricTagConfigurationUpdateRequest
        :rtype: MetricTagConfigurationResponse
        """
        kwargs: Dict[str, Any] = {}
        kwargs["metric_name"] = metric_name

        kwargs["body"] = body

        return self._update_tag_configuration_endpoint.call_with_http_info(**kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelSimple,
    cached_property,
)

from typing import ClassVar


class TopologyQueryDataSource(ModelSimple):
    """
    Name of the data source

    :param value: Must be one of ["data_streams", "service_map"].
    :type value: str
    """

    allowed_values = {
        "data_streams",
        "service_map",
    }
    DATA_STREAMS: ClassVar["TopologyQueryDataSource"]
    SERVICE_MAP: ClassVar["TopologyQueryDataSource"]

    @cached_property
    def openapi_types(_):
        return {
            "value": (str,),
        }


TopologyQueryDataSource.DATA_STREAMS = TopologyQueryDataSource("data_streams")
TopologyQueryDataSource.SERVICE_MAP = TopologyQueryDataSource("service_map")



# coding=utf-8
"""Define basic fixtures."""

import json
import os
import pathlib
import re
import warnings
import zlib
from collections import defaultdict

import pytest
from dateutil.relativedelta import relativedelta
from jinja2 import Environment, FileSystemLoader, Template
from pytest_bdd import given, parsers, then, when

from generator import openapi

from generator.formatter import format_parameters, format_data_with_schema, safe_snake_case, snake_case


MODIFIED_FEATURES = {
    pathlib.Path(p).resolve()
    for p in os.getenv("BDD_MODIFIED_FEATURES", "").split(" ")
    if p
}

ROOT_PATH = pathlib.Path(__file__).parent.parent

PATTERN_ALPHANUM = re.compile(r"[^A-Za-z0-9]+")


if os.getenv("CI") is not None:

    def formatwarning(message, category, filename, lineno, line=None):
        p = pathlib.Path(filename)
        if p.is_absolute() and filename.startswith(str(ROOT_PATH)):
            p = p.relative_to(ROOT_PATH)
        return f"\n::warning file={p},line={lineno}::{message}\n"

    warnings.formatwarning = formatwarning


def pytest_bdd_before_scenario(request, feature, scenario):
    if MODIFIED_FEATURES:
        current = pathlib.Path(scenario.feature.filename).resolve()
        if current not in MODIFIED_FEATURES:
            pytest.skip(
                f"Feature file {scenario.feature.filename} has not been modified"
            )


def lookup(value, path):
    result = value
    for dot_path in path.split("."):
        for part in dot_path.split("["):
            if "]" in part:
                index = int(part[: part.index("]")])
                result = result[index]
            else:
                result = result[snake_case(part)]
    return result


def json_dumps(*args, **kwargs):
    return json.dumps(*args, **kwargs)


class FloatEncoder(json.JSONEncoder):
    """Ensure that floats have decimal point and no trailing zeros."""

    def encode(self, obj):
        result = super().encode(obj)
        if isinstance(obj, float):
            if "." in result and result.endswith("0"):
                result = result.rstrip("0")
            result = result.rstrip(".")
            return result
        return result


JINJA_ENV = Environment(loader=FileSystemLoader(pathlib.Path(__file__).parent / "src" / "generator" / "templates"))
JINJA_ENV.filters["tojson"] = json.dumps
JINJA_ENV.filters["snake_case"] = snake_case
JINJA_ENV.filters["safe_snake_case"] = safe_snake_case
JINJA_ENV.globals["format_data_with_schema"] = format_data_with_schema
JINJA_ENV.globals["format_parameters"] = format_parameters

PYTHON_EXAMPLE_J2 = JINJA_ENV.get_template("example.j2")


def pytest_bdd_after_scenario(request, feature, scenario):
    try:
        operation_specs = request.getfixturevalue("operation_specs")
        version = request.getfixturevalue("api_version")
        context = request.getfixturevalue("context")
    except Exception:
        return
    operation_id = context["api_request"]["operation_id"]

    status_code = context["status_code"]
    if status_code >= 300:
        return

    operation_spec = operation_specs[version][operation_id]
    response_spec = operation_spec.spec["responses"][str(status_code)]
    group_name = "-".join(operation_spec.spec["tags"][0].split(" ")).lower()
    context["api_response"] = response_spec

    unique_suffix = ""
    scenario_name = f"{operation_spec.spec['summary']} returns \"{response_spec['description']}\" response"
    if scenario_name != scenario.name:
        unique_suffix = "_" + str(zlib.adler32(scenario.name.encode("utf-8")))

    code_examples = request.getfixturevalue("code_examples")
    code_examples.setdefault(version, {}).setdefault(operation_id, []).append(
        {
            "group": "_".join(operation_spec.spec["tags"][0].split(" ")).lower(),
            "suffix": unique_suffix,
            "description": scenario.name,
        }
    )

    data = PYTHON_EXAMPLE_J2.render(
        context=context,
        version=version,
        scenario=scenario,
        operation_spec=operation_spec.spec,
    )

    output = (
        ROOT_PATH
        / "examples"
        / version
        / group_name
        / f"{operation_id}{unique_suffix}.py"
    )
    output.parent.mkdir(parents=True, exist_ok=True)

    with output.open("w") as f:
        f.write(data)


def pytest_bdd_apply_tag(tag, function):
    """Register tags as custom markers and skip test for '@skip' ones."""
    skip_tags = {}
    if tag in skip_tags:
        marker = pytest.mark.skip(reason=f"skipped because '{tag}' in {skip_tags}")
        marker(function)
        return True
    return False


def _get_prefix(request):
    test_class = request.cls
    if test_class:
        main = "{}.{}".format(test_class.__name__, request.node.name)
    else:
        base_name = request.node.__scenario_report__.scenario.name
        main = PATTERN_ALPHANUM.sub("_", base_name)[:100]
    prefix = "Example-"
    return f"{prefix}{main}"


@pytest.fixture(scope="session")
def code_examples():
    return {}


@pytest.fixture(scope="session")
def notifications():
    with (ROOT_PATH / "notifications.json").open() as f:
        return json.load(f)


@pytest.fixture
def api_version(request):
    path = pathlib.Path(request.node.__scenario_report__.scenario.feature.filename)
    return path.parent.parent.name


@pytest.fixture
def unique(request):
    return _get_prefix(request)


TIME_FORMATTER = {
    "now": "datetime.now()",
    "timestamp": "{sret}.timestamp()",
    "isoformat": "{sret}",
    "units": {
        "s": "({sret} + relativedelta(seconds={num}))",
        "m": "({sret} + relativedelta(minutes={num}))",
        "h": "({sret} + relativedelta(hours={num}))",
        "d": "({sret} + relativedelta(days={num}))",
        "M": "({sret} + relativedelta(months={num}))",
        "y": "({sret} + relativedelta(years={num}))",
    },
}


def relative_time(imports, calls, freezed_time, iso):
    time_re = re.compile(r"now( *([+-]) *(\d+)([smhdMy]))?")

    def func(arg):
        imports["datetime"].add("datetime")
        sret = TIME_FORMATTER["now"]
        ret = freezed_time
        m = time_re.match(arg)
        if m:
            if m.group(1):
                sign = m.group(2)
                num = int(sign + m.group(3))
                unit = m.group(4)
                if unit == "s":
                    ret += relativedelta(seconds=num)
                elif unit == "m":
                    ret += relativedelta(minutes=num)
                elif unit == "h":
                    ret += relativedelta(hours=num)
                elif unit == "d":
                    ret += relativedelta(days=num)
                elif unit == "M":
                    ret += relativedelta(months=num)
                elif unit == "y":
                    ret += relativedelta(years=num)
                else:
                    raise ValueError(f"Unknown unit {unit}")
                sret = TIME_FORMATTER["units"][unit].format(sret=sret, num=num)
                imports["dateutil.relativedelta"].add("relativedelta")

            if iso:
                return (
                    ret.isoformat(timespec="seconds"),
                    TIME_FORMATTER["isoformat"].format(sret=sret),
                )
            return int(ret.timestamp()), TIME_FORMATTER["timestamp"].format(sret=sret)
        return "", ""

    def store_calls(arg):
        result, value = func(arg)
        calls[result] = value
        return result

    return store_calls


@pytest.fixture
def context(request, unique, freezed_time):
    """
    Return a mapping with all defined fixtures, all objects created by `given` steps,
    and the undo operations to perform after a test scenario.
    """
    prefix = _get_prefix(request)

    class MarkUsed(dict):
        def __init__(self, *args, **kwargs):
            dict.__init__(self, *args, **kwargs)
            self.__used_keys__ = set()

        def __getitem__(self, key):
            value = super().__getitem__(key)
            self.__used_keys__.add(value)
            return value

        def is_used(self, key):
            return key in self.__used_keys__

    replace_values = MarkUsed()
    imports = defaultdict(set)
    given = defaultdict(dict)

    ctx = {
        "undo_operations": [],
        "unique": unique,
        "unique_lower": unique.lower(),
        "unique_upper": unique.upper(),
        "unique_alnum": PATTERN_ALPHANUM.sub("", unique),
        "unique_lower_alnum": PATTERN_ALPHANUM.sub("", unique).lower(),
        "unique_upper_alnum": PATTERN_ALPHANUM.sub("", unique).upper(),
        "timestamp": relative_time(imports, replace_values, freezed_time, False),
        "timeISO": relative_time(imports, replace_values, freezed_time, True),
        "_asserts": [],
        "_replace_values": replace_values,
        "_imports": imports,
        "_given": given,
        "_given_parameters": set(),
        "_given_requests": {},
        "_key_to_json_path": defaultdict(dict),
        "_enable_operations": set(),
        "_local_variables": {
            "unique": prefix + "_{{ timestamp(0, s) }}",
            "unique_lower": prefix.lower() + "_{{ timestamp(0, s) }}",
            "unique_upper": prefix.upper() + "_{{ timestamp(0, s) }}",
            "unique_alnum": re.sub(r"[^A-Za-z0-9]+", "", prefix)
            + "{{ timestamp(0, s) }}",
            "unique_lower_alnum": re.sub(r"[^A-Za-z0-9]+", "", prefix).lower()
            + "{{ timestamp(0, s) }}",
            "unique_upper_alnum": re.sub(r"[^A-Za-z0-9]+", "", prefix).upper()
            + "{{ timestamp(0, s) }}",
        },
    }

    yield ctx


@pytest.fixture
def freezed_time():
    from dateutil import parser

    return parser.isoparse("2021-11-11T11:11:11.111111+00:00")


@given('a valid "apiKeyAuth" key in the system')
def a_valid_api_key(context):
    """a valid API key."""
    context["has_api_key"] = True


@given('a valid "appKeyAuth" key in the system')
def a_valid_application_key(context):
    """a valid Application key."""
    context["has_application_key"] = True


@pytest.fixture(scope="module")
def specs():
    result = {}
    for f in (ROOT_PATH / ".generator" / "schemas").rglob("openapi.yaml"):
        version = f.parent.name
        result[version] = openapi.load(f)

    return result


@pytest.fixture(scope="module")
def operation_specs(specs):
    by_operation = {}

    for version, spec in specs.items():
        by_operation[version] = {}
        for path in spec["paths"]:
            for method, operation in spec["paths"][path].items():
                by_operation[version][operation["operationId"]] = openapi.Operation(
                    operation["operationId"], operation, method, path
                )

    return by_operation


@pytest.fixture(scope="module")
def undo_operations():
    result = {}

    for f in (ROOT_PATH / "tests").rglob("undo.json"):
        version = f.parent.parent.name
        with f.open() as fp:
            result[version] = json.load(fp)

    return result


@given(parsers.parse('an instance of "{name}" API'))
def api(context, api_version, specs, name):
    """Return an API instance."""
    assert name in {tag["name"].replace(" ", "") for tag in specs[api_version]["tags"]}
    context["api_instance"] = {"name": name}


@given(parsers.parse('operation "{name}" enabled'))
def operation_enabled(context, name):
    """Enable the unstable operation specific in the clause."""
    context["_enable_operations"].add(name)


@given(parsers.parse('new "{name}" request'), target_fixture="operation_id")
def api_request(context, operation_specs, api_version, name):
    """Call an endpoint."""
    context["api_request"] = {"operation_id": name, "kwargs": {}}
    operation_spec = operation_specs[api_version][name]
    try:
        context["api_request"]["schema"] = operation_spec.request()
    except KeyError:
        pass
    return name


@given(parsers.parse("body with value {data}"))
def request_body(request, context, data):
    """Set request body."""
    tpl = Template(data).render(**context)
    context["body"] = {
        "tpl": data,
        "value": json.loads(tpl),
    }


@given(parsers.parse('body from file "{path}"'))
def request_body_from_file(request, context, path, api_version):
    """Set request body."""
    body_file = ROOT_PATH / "tests" / api_version / "features" / path
    with body_file.open() as f:
        data = f.read()
    tpl = Template(data).render(**context)
    context["body"] = {
        "tpl": data,
        "value": json.loads(tpl),
    }


@given(parsers.parse('request contains "{name}" parameter from "{path}"'))
def request_parameter(context, operation_id, api_version, operation_specs, name, path):
    """Set request parameter."""
    try:
        value = lookup(context, path)
        value = value.value(value)  # trigger replacement recording
    except KeyError:
        if path != "REPLACE.ME":
            raise

        parameters = operation_specs[api_version][operation_id].spec["parameters"]
        for parameter in parameters:
            if parameter["name"] == name:
                schema = parameter.get("schema", {})
                value = schema.get("example", schema.get("default"))
                if value is None:
                    type_ = schema.get("type")
                    format_ = schema.get("format")
                    value = {
                        "string": {
                            "date-time": "2021-11-11T11:11:11.111+00:00",
                            None: name,
                        },
                        "integer": {
                            "int32": 1,
                            "int64": 9223372036854775807,
                            None: 1,
                        },
                        "array": {
                            None: [],
                        },
                    }[type_][format_]
                break

    context["api_request"]["kwargs"][name] = {
        "path": path,
        "value": value,
    }


@given(parsers.parse('request contains "{name}" parameter with value {value}'))
def request_parameter_with_value(context, name, value):
    """Set request parameter."""
    tpl = Template(value).render(**context)
    context["api_request"]["kwargs"][name] = {
        "tpl": value,
        "value": json.loads(tpl),
    }


def build_given(version, operation):
    def wrapper(context, operation_specs):

        undo_prefix = [operation["key"]]

        def make_path(keys):
            result = operation["key"] + "_" + "_".join(str(k) for k in keys)
            return result.upper()

        # store response in fixtures
        def record_value(schema):
            if "default" in schema.spec and "enum" in schema.spec:
                return schema.spec["default"]

            value = openapi.generate_value(schema)
            key = make_path(schema.keys)
            context["_given"][operation["step"]][key] = schema.spec
            if context["_replace_values"].get(value, key) != key:
                value = openapi.generate_value(schema, use_random=True, prefix=key)

            context["_replace_values"][value] = key
            keys = (
                [operation["source"]] + list(schema.keys)
                if "source" in operation
                else schema.keys
            )
            json_path = "".join(
                f"[{k}]" if isinstance(k, int) else f".{k}" for k in keys
            ).strip(".")
            assert (
                context["_key_to_json_path"][operation["key"]].get(key, json_path)
                == json_path
            )
            context["_key_to_json_path"][operation["key"]][key] = json_path
            return value

        operation_spec = operation_specs[version][operation["operationId"]]
        response_spec = operation_spec.response()
        if "source" in operation:
            response_spec = lookup(response_spec, operation["source"])
            undo_prefix.extend(response_spec.keys)

        response_spec.keys = ()
        response_spec.value = record_value
        response_spec.__source__ = operation.get("source")

        context[operation["key"]] = response_spec

    return wrapper


for f in (ROOT_PATH / "tests").rglob("given.json"):
    version = f.parent.parent.name
    with f.open() as fp:
        for settings in json.load(fp):
            given(settings["step"])(build_given(version, settings))


@when("the request is sent")
def execute_request(context):
    """Execute the prepared request."""
    assert context["api_request"]["operation_id"] is not None


@when("the request with pagination is sent")
def execute_request_with_pagination(context):
    context["pagination"] = True


@then(parsers.parse("the response status is {status:d} {description}"))
def the_status_is(context, status, description):
    """Check the status."""
    context["status_code"] = status


@then(parsers.parse('the response "{response_path}" is equal to {value}'))
def expect_equal(context, response_path, value):
    """Compare a response attribute to a value."""


@then(
    parsers.parse(
        'the response "{response_path}" has the same value as "{fixture_path}"'
    )
)
def expect_equal_value(context, response_path, fixture_path):
    """Compare a response attribute to another attribute."""


@then(parsers.parse('the response "{response_path}" has length {fixture_length:d}'))
def expect_equal_length(context, response_path, fixture_length):
    """Check the length of a response attribute."""


@then(parsers.parse("the response has {fixture_length:d} items"))
def expect_equal_response_items(context, fixture_length):
    """Check the size of a response."""


@then(parsers.parse('the response "{response_path}" is false'))
def expect_false(context, response_path):
    """Check that a response attribute is false."""



"""
Create a new dashboard with template variable defaults returns "OK" response
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.dashboards_api import DashboardsApi
from datadog_api_client.v1.model.dashboard import Dashboard
from datadog_api_client.v1.model.dashboard_layout_type import DashboardLayoutType
from datadog_api_client.v1.model.dashboard_reflow_type import DashboardReflowType
from datadog_api_client.v1.model.dashboard_template_variable import DashboardTemplateVariable
from datadog_api_client.v1.model.host_map_request import HostMapRequest
from datadog_api_client.v1.model.host_map_widget_definition import HostMapWidgetDefinition
from datadog_api_client.v1.model.host_map_widget_definition_requests import HostMapWidgetDefinitionRequests
from datadog_api_client.v1.model.host_map_widget_definition_type import HostMapWidgetDefinitionType
from datadog_api_client.v1.model.widget import Widget

body = Dashboard(
    description=None,
    is_read_only=False,
    layout_type=DashboardLayoutType.ORDERED,
    notify_list=[],
    reflow_type=DashboardReflowType.AUTO,
    restricted_roles=[],
    template_variables=[
        DashboardTemplateVariable(
            available_values=[
                "my-host",
                "host1",
                "host2",
            ],
            defaults=[
                "my-host",
            ],
            name="host1",
            prefix="host",
        ),
    ],
    title="",
    widgets=[
        Widget(
            definition=HostMapWidgetDefinition(
                requests=HostMapWidgetDefinitionRequests(
                    fill=HostMapRequest(
                        q="avg:system.cpu.user{*}",
                    ),
                ),
                type=HostMapWidgetDefinitionType.HOSTMAP,
            ),
        ),
    ],
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = DashboardsApi(api_client)
    response = api_instance.create_dashboard(body=body)

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Any, Dict

from datadog_api_client.api_client import ApiClient, Endpoint as _Endpoint
from datadog_api_client.v1.model.webhooks_integration_custom_variable_response import (
    WebhooksIntegrationCustomVariableResponse,
)
from datadog_api_client.v1.model.webhooks_integration_custom_variable import WebhooksIntegrationCustomVariable
from datadog_api_client.v1.model.webhooks_integration_custom_variable_update_request import (
    WebhooksIntegrationCustomVariableUpdateRequest,
)
from datadog_api_client.v1.model.webhooks_integration import WebhooksIntegration
from datadog_api_client.v1.model.webhooks_integration_update_request import WebhooksIntegrationUpdateRequest


class WebhooksIntegrationApi:
    """
    Configure your Datadog-Webhooks integration directly through the Datadog API.
    For more information about the Datadog-Webhooks integration,
    see the `integration page <https://docs.datadoghq.com/integrations/webhooks>`_.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        self._create_webhooks_integration_endpoint = _Endpoint(
            settings={
                "response_type": (WebhooksIntegration,),
                "auth": ["apiKeyAuth", "appKeyAuth"],
                "endpoint_path": "/api/v1/integration/webhooks/configuration/webhooks",
                "operation_id": "create_webhooks_integration",
                "http_method": "POST",
                "version": "v1",
                "servers": None,
            },
            params_map={
                "body": {
                    "required": True,
                    "openapi_types": (WebhooksIntegration,),
                    "location": "body",
                },
            },
            headers_map={"accept": ["application/json"], "content_type": ["application/json"]},
            api_client=api_client,
        )

        self._create_webhooks_integration_custom_variable_endpoint = _Endpoint(
            settings={
                "response_type": (WebhooksIntegrationCustomVariableResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth"],
                "endpoint_path": "/api/v1/integration/webhooks/configuration/custom-variables",
                "operation_id": "create_webhooks_integration_custom_variable",
                "http_method": "POST",
                "version": "v1",
                "servers": None,
            },
            params_map={
                "body": {
                    "required": True,
                    "openapi_types": (WebhooksIntegrationCustomVariable,),
                    "location": "body",
                },
            },
            headers_map={"accept": ["application/json"], "content_type": ["application/json"]},
            api_client=api_client,
        )

        self._delete_webhooks_integration_endpoint = _Endpoint(
            settings={
                "response_type": None,
                "auth": ["apiKeyAuth", "appKeyAuth"],
                "endpoint_path": "/api/v1/integration/webhooks/configuration/webhooks/{webhook_name}",
                "operation_id": "delete_webhooks_integration",
                "http_method": "DELETE",
                "version": "v1",
                "servers": None,
            },
            params_map={
                "webhook_name": {
                    "required": True,
                    "openapi_types": (str,),
                    "attribute": "webhook_name",
                    "location": "path",
                },
            },
            headers_map={
                "accept": ["*/*"],
                "content_type": [],
            },
            api_client=api_client,
        )

        self._delete_webhooks_integration_custom_variable_endpoint = _Endpoint(
            settings={
                "response_type": None,
                "auth": ["apiKeyAuth", "appKeyAuth"],
                "endpoint_path": "/api/v1/integration/webhooks/configuration/custom-variables/{custom_variable_name}",
                "operation_id": "delete_webhooks_integration_custom_variable",
                "http_method": "DELETE",
                "version": "v1",
                "servers": None,
            },
            params_map={
                "custom_variable_name": {
                    "required": True,
                    "openapi_types": (str,),
                    "attribute": "custom_variable_name",
                    "location": "path",
                },
            },
            headers_map={
                "accept": ["*/*"],
                "content_type": [],
            },
            api_client=api_client,
        )

        self._get_webhooks_integration_endpoint = _Endpoint(
            settings={
                "response_type": (WebhooksIntegration,),
                "auth": ["apiKeyAuth", "appKeyAuth"],
                "endpoint_path": "/api/v1/integration/webhooks/configuration/webhooks/{webhook_name}",
                "operation_id": "get_webhooks_integration",
                "http_method": "GET",
                "version": "v1",
                "servers": None,
            },
            params_map={
                "webhook_name": {
                    "required": True,
                    "openapi_types": (str,),
                    "attribute": "webhook_name",
                    "location": "path",
                },
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )

        self._get_webhooks_integration_custom_variable_endpoint = _Endpoint(
            settings={
                "response_type": (WebhooksIntegrationCustomVariableResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth"],
                "endpoint_path": "/api/v1/integration/webhooks/configuration/custom-variables/{custom_variable_name}",
                "operation_id": "get_webhooks_integration_custom_variable",
                "http_method": "GET",
                "version": "v1",
                "servers": None,
            },
            params_map={
                "custom_variable_name": {
                    "required": True,
                    "openapi_types": (str,),
                    "attribute": "custom_variable_name",
                    "location": "path",
                },
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )

        self._update_webhooks_integration_endpoint = _Endpoint(
            settings={
                "response_type": (WebhooksIntegration,),
                "auth": ["apiKeyAuth", "appKeyAuth"],
                "endpoint_path": "/api/v1/integration/webhooks/configuration/webhooks/{webhook_name}",
                "operation_id": "update_webhooks_integration",
                "http_method": "PUT",
                "version": "v1",
                "servers": None,
            },
            params_map={
                "webhook_name": {
                    "required": True,
                    "openapi_types": (str,),
                    "attribute": "webhook_name",
                    "location": "path",
                },
                "body": {
                    "required": True,
                    "openapi_types": (WebhooksIntegrationUpdateRequest,),
                    "location": "body",
                },
            },
            headers_map={"accept": ["application/json"], "content_type": ["application/json"]},
            api_client=api_client,
        )

        self._update_webhooks_integration_custom_variable_endpoint = _Endpoint(
            settings={
                "response_type": (WebhooksIntegrationCustomVariableResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth"],
                "endpoint_path": "/api/v1/integration/webhooks/configuration/custom-variables/{custom_variable_name}",
                "operation_id": "update_webhooks_integration_custom_variable",
                "http_method": "PUT",
                "version": "v1",
                "servers": None,
            },
            params_map={
                "custom_variable_name": {
                    "required": True,
                    "openapi_types": (str,),
                    "attribute": "custom_variable_name",
                    "location": "path",
                },
                "body": {
                    "required": True,
                    "openapi_types": (WebhooksIntegrationCustomVariableUpdateRequest,),
                    "location": "body",
                },
            },
            headers_map={"accept": ["application/json"], "content_type": ["application/json"]},
            api_client=api_client,
        )

    def create_webhooks_integration(
        self,
        body: WebhooksIntegration,
    ) -> WebhooksIntegration:
        """Create a webhooks integration.

        Creates an endpoint with the name ``<WEBHOOK_NAME>``.

        :param body: Create a webhooks integration request body.
        :type body: WebhooksIntegration
        :rtype: WebhooksIntegration
        """
        kwargs: Dict[str, Any] = {}
        kwargs["body"] = body

        return self._create_webhooks_integration_endpoint.call_with_http_info(**kwargs)

    def create_webhooks_integration_custom_variable(
        self,
        body: WebhooksIntegrationCustomVariable,
    ) -> WebhooksIntegrationCustomVariableResponse:
        """Create a custom variable.

        Creates an endpoint with the name ``<CUSTOM_VARIABLE_NAME>``.

        :param body: Define a custom variable request body.
        :type body: WebhooksIntegrationCustomVariable
        :rtype: WebhooksIntegrationCustomVariableResponse
        """
        kwargs: Dict[str, Any] = {}
        kwargs["body"] = body

        return self._create_webhooks_integration_custom_variable_endpoint.call_with_http_info(**kwargs)

    def delete_webhooks_integration(
        self,
        webhook_name: str,
    ) -> None:
        """Delete a webhook.

        Deletes the endpoint with the name ``<WEBHOOK NAME>``.

        :param webhook_name: The name of the webhook.
        :type webhook_name: str
        :rtype: None
        """
        kwargs: Dict[str, Any] = {}
        kwargs["webhook_name"] = webhook_name

        return self._delete_webhooks_integration_endpoint.call_with_http_info(**kwargs)

    def delete_webhooks_integration_custom_variable(
        self,
        custom_variable_name: str,
    ) -> None:
        """Delete a custom variable.

        Deletes the endpoint with the name ``<CUSTOM_VARIABLE_NAME>``.

        :param custom_variable_name: The name of the custom variable.
        :type custom_variable_name: str
        :rtype: None
        """
        kwargs: Dict[str, Any] = {}
        kwargs["custom_variable_name"] = custom_variable_name

        return self._delete_webhooks_integration_custom_variable_endpoint.call_with_http_info(**kwargs)

    def get_webhooks_integration(
        self,
        webhook_name: str,
    ) -> WebhooksIntegration:
        """Get a webhook integration.

        Gets the content of the webhook with the name ``<WEBHOOK_NAME>``.

        :param webhook_name: The name of the webhook.
        :type webhook_name: str
        :rtype: WebhooksIntegration
        """
        kwargs: Dict[str, Any] = {}
        kwargs["webhook_name"] = webhook_name

        return self._get_webhooks_integration_endpoint.call_with_http_info(**kwargs)

    def get_webhooks_integration_custom_variable(
        self,
        custom_variable_name: str,
    ) -> WebhooksIntegrationCustomVariableResponse:
        """Get a custom variable.

        Shows the content of the custom variable with the name ``<CUSTOM_VARIABLE_NAME>``.

        If the custom variable is secret, the value does not return in the
        response payload.

        :param custom_variable_name: The name of the custom variable.
        :type custom_variable_name: str
        :rtype: WebhooksIntegrationCustomVariableResponse
        """
        kwargs: Dict[str, Any] = {}
        kwargs["custom_variable_name"] = custom_variable_name

        return self._get_webhooks_integration_custom_variable_endpoint.call_with_http_info(**kwargs)

    def update_webhooks_integration(
        self,
        webhook_name: str,
        body: WebhooksIntegrationUpdateRequest,
    ) -> WebhooksIntegration:
        """Update a webhook.

        Updates the endpoint with the name ``<WEBHOOK_NAME>``.

        :param webhook_name: The name of the webhook.
        :type webhook_name: str
        :param body: Update an existing Datadog-Webhooks integration.
        :type body: WebhooksIntegrationUpdateRequest
        :rtype: WebhooksIntegration
        """
        kwargs: Dict[str, Any] = {}
        kwargs["webhook_name"] = webhook_name

        kwargs["body"] = body

        return self._update_webhooks_integration_endpoint.call_with_http_info(**kwargs)

    def update_webhooks_integration_custom_variable(
        self,
        custom_variable_name: str,
        body: WebhooksIntegrationCustomVariableUpdateRequest,
    ) -> WebhooksIntegrationCustomVariableResponse:
        """Update a custom variable.

        Updates the endpoint with the name ``<CUSTOM_VARIABLE_NAME>``.

        :param custom_variable_name: The name of the custom variable.
        :type custom_variable_name: str
        :param body: Update an existing custom variable request body.
        :type body: WebhooksIntegrationCustomVariableUpdateRequest
        :rtype: WebhooksIntegrationCustomVariableResponse
        """
        kwargs: Dict[str, Any] = {}
        kwargs["custom_variable_name"] = custom_variable_name

        kwargs["body"] = body

        return self._update_webhooks_integration_custom_variable_endpoint.call_with_http_info(**kwargs)



# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import difflib
import os
import sys

from sphinx.domains.python import PythonDomain

sys.path.insert(0, os.path.abspath("../src"))


# -- Project information -----------------------------------------------------

project = "Datadog API Client for Python"
copyright = "2022, Datadog"
author = "Datadog"

# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = ["m2r2", "sphinx.ext.autodoc"]

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "sphinx_material"

html_logo = "assets/images/logo.svg"
html_favicon = "assets/images/favicon.ico"

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ["assets"]

html_sidebars = {
    "**": ["globaltoc.html", "localtoc.html", "searchbox.html"]
}

html_theme_options = {
    # Set the name of the project to appear in the navigation.
    "nav_title": "Datadog API Client for Python",
    # Set the color and the accent color
    "color_primary": "deep-purple",
    "color_accent": "deep-purple",
    # Set the repo location to get a badge with stats
    "repo_url": "https://github.com/DataDog/datadog-api-client-python/",
    "repo_name": "datadog-api-client-python",
    # Visible levels of the global TOC; -1 means unlimited
    "globaltoc_depth": 3,
    # If False, expand all TOC entries
    "globaltoc_collapse": False,
    # If True, show hidden TOC entries
    "globaltoc_includehidden": False,
    # No version menu
    "version_dropdown": False,
}

add_module_names = False

autoclass_content = "both"


def find_obj(domain, env, modname, classname, name, type, searchmode=0):
    # skip parens
    if name[-2:] == '()':
        name = name[:-2]

    if not name:
        return []

    matches = []

    newname = None
    if searchmode == 1:
        if type is None:
            objtypes = list(domain.object_types)
        else:
            objtypes = domain.objtypes_for_role(type)
        if objtypes is not None:
            if modname and classname:
                fullname = modname + '.' + classname + '.' + name
                if fullname in domain.objects and domain.objects[fullname].objtype in objtypes:
                    newname = fullname
            if not newname:
                if modname and modname + '.' + name in domain.objects and \
                   domain.objects[modname + '.' + name].objtype in objtypes:
                    newname = modname + '.' + name
                elif name in domain.objects and domain.objects[name].objtype in objtypes:
                    newname = name
                else:
                    # "fuzzy" searching mode
                    searchname = '.' + name
                    matches = [(oname, domain.objects[oname]) for oname in domain.objects
                               if oname.endswith(searchname) and
                               domain.objects[oname].objtype in objtypes]
                    if len(matches) > 1:
                        close_match = difflib.get_close_matches(modname, [m[0] for m in matches], n=1, cutoff=0.1)
                        if close_match:
                            matches = [m for m in matches if m[0] == close_match[0]]
    else:
        # NOTE: searching for exact match, object type is not considered
        if name in domain.objects:
            newname = name
        elif type == 'mod':
            # only exact matches allowed for modules
            return []
        elif classname and classname + '.' + name in domain.objects:
            newname = classname + '.' + name
        elif modname and modname + '.' + name in domain.objects:
            newname = modname + '.' + name
        elif modname and classname and \
                modname + '.' + classname + '.' + name in domain.objects:
            newname = modname + '.' + classname + '.' + name
    if newname is not None:
        matches.append((newname, domain.objects[newname]))
    return matches


PythonDomain.find_obj = find_obj



"""
Create a detection rule with type 'impossible_travel' returns "OK" response
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v2.api.security_monitoring_api import SecurityMonitoringApi
from datadog_api_client.v2.model.security_monitoring_rule_case_create import SecurityMonitoringRuleCaseCreate
from datadog_api_client.v2.model.security_monitoring_rule_detection_method import SecurityMonitoringRuleDetectionMethod
from datadog_api_client.v2.model.security_monitoring_rule_evaluation_window import (
    SecurityMonitoringRuleEvaluationWindow,
)
from datadog_api_client.v2.model.security_monitoring_rule_impossible_travel_options import (
    SecurityMonitoringRuleImpossibleTravelOptions,
)
from datadog_api_client.v2.model.security_monitoring_rule_keep_alive import SecurityMonitoringRuleKeepAlive
from datadog_api_client.v2.model.security_monitoring_rule_max_signal_duration import (
    SecurityMonitoringRuleMaxSignalDuration,
)
from datadog_api_client.v2.model.security_monitoring_rule_options import SecurityMonitoringRuleOptions
from datadog_api_client.v2.model.security_monitoring_rule_query_aggregation import (
    SecurityMonitoringRuleQueryAggregation,
)
from datadog_api_client.v2.model.security_monitoring_rule_severity import SecurityMonitoringRuleSeverity
from datadog_api_client.v2.model.security_monitoring_rule_type_create import SecurityMonitoringRuleTypeCreate
from datadog_api_client.v2.model.security_monitoring_standard_rule_create_payload import (
    SecurityMonitoringStandardRuleCreatePayload,
)
from datadog_api_client.v2.model.security_monitoring_standard_rule_query import SecurityMonitoringStandardRuleQuery

body = SecurityMonitoringStandardRuleCreatePayload(
    queries=[
        SecurityMonitoringStandardRuleQuery(
            aggregation=SecurityMonitoringRuleQueryAggregation.GEO_DATA,
            group_by_fields=[
                "@usr.id",
            ],
            distinct_fields=[],
            metric="@network.client.geoip",
            query="*",
        ),
    ],
    cases=[
        SecurityMonitoringRuleCaseCreate(
            name="",
            status=SecurityMonitoringRuleSeverity.INFO,
            notifications=[],
        ),
    ],
    has_extended_title=True,
    message="test",
    is_enabled=True,
    options=SecurityMonitoringRuleOptions(
        max_signal_duration=SecurityMonitoringRuleMaxSignalDuration.ONE_DAY,
        evaluation_window=SecurityMonitoringRuleEvaluationWindow.FIFTEEN_MINUTES,
        keep_alive=SecurityMonitoringRuleKeepAlive.ONE_HOUR,
        detection_method=SecurityMonitoringRuleDetectionMethod.IMPOSSIBLE_TRAVEL,
        impossible_travel_options=SecurityMonitoringRuleImpossibleTravelOptions(
            baseline_user_locations=False,
        ),
    ),
    name="Example-Create_a_detection_rule_with_type_impossible_travel_returns_OK_response",
    type=SecurityMonitoringRuleTypeCreate.LOG_DETECTION,
    tags=[],
    filters=[],
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = SecurityMonitoringApi(api_client)
    response = api_instance.create_security_monitoring_rule(body=body)

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import List, Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v1.model.logs_status_remapper_type import LogsStatusRemapperType


class LogsStatusRemapper(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v1.model.logs_status_remapper_type import LogsStatusRemapperType

        return {
            "is_enabled": (bool,),
            "name": (str,),
            "sources": ([str],),
            "type": (LogsStatusRemapperType,),
        }

    attribute_map = {
        "is_enabled": "is_enabled",
        "name": "name",
        "sources": "sources",
        "type": "type",
    }

    def __init__(
        self_,
        sources: List[str],
        type: LogsStatusRemapperType,
        is_enabled: Union[bool, UnsetType] = unset,
        name: Union[str, UnsetType] = unset,
        **kwargs,
    ):
        """
        Use this Processor if you want to assign some attributes as the official status.

        Each incoming status value is mapped as follows.

        * Integers from 0 to 7 map to the Syslog severity standards
        * Strings beginning with ``emerg`` or f (case-insensitive) map to ``emerg`` (0)
        * Strings beginning with ``a`` (case-insensitive) map to ``alert`` (1)
        * Strings beginning with ``c`` (case-insensitive) map to ``critical`` (2)
        * Strings beginning with ``err`` (case-insensitive) map to ``error`` (3)
        * Strings beginning with ``w`` (case-insensitive) map to ``warning`` (4)
        * Strings beginning with ``n`` (case-insensitive) map to ``notice`` (5)
        * Strings beginning with ``i`` (case-insensitive) map to ``info`` (6)
        * Strings beginning with ``d`` , ``trace`` or ``verbose`` (case-insensitive) map to ``debug`` (7)
        * Strings beginning with ``o`` or matching ``OK`` or ``Success`` (case-insensitive) map to OK
        *
          All others map to ``info`` (6)

          **Note:** If multiple log status remapper processors can be applied to a given log,
          only the first one (according to the pipelines order) is taken into account.

        :param is_enabled: Whether or not the processor is enabled.
        :type is_enabled: bool, optional

        :param name: Name of the processor.
        :type name: str, optional

        :param sources: Array of source attributes.
        :type sources: [str]

        :param type: Type of logs status remapper.
        :type type: LogsStatusRemapperType
        """
        if is_enabled is not unset:
            kwargs["is_enabled"] = is_enabled
        if name is not unset:
            kwargs["name"] = name
        super().__init__(kwargs)

        self_.sources = sources
        self_.type = type



"""
Create a service account returns "OK" response
"""

from os import environ
from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v2.api.users_api import UsersApi
from datadog_api_client.v2.model.relationship_to_role_data import RelationshipToRoleData
from datadog_api_client.v2.model.relationship_to_roles import RelationshipToRoles
from datadog_api_client.v2.model.roles_type import RolesType
from datadog_api_client.v2.model.service_account_create_attributes import ServiceAccountCreateAttributes
from datadog_api_client.v2.model.service_account_create_data import ServiceAccountCreateData
from datadog_api_client.v2.model.service_account_create_request import ServiceAccountCreateRequest
from datadog_api_client.v2.model.user_relationships import UserRelationships
from datadog_api_client.v2.model.users_type import UsersType

# there is a valid "role" in the system
ROLE_DATA_ID = environ["ROLE_DATA_ID"]

body = ServiceAccountCreateRequest(
    data=ServiceAccountCreateData(
        type=UsersType.USERS,
        attributes=ServiceAccountCreateAttributes(
            name="Test API Client",
            email="Example-Create_a_service_account_returns_OK_response@datadoghq.com",
            service_account=True,
        ),
        relationships=UserRelationships(
            roles=RelationshipToRoles(
                data=[
                    RelationshipToRoleData(
                        id=ROLE_DATA_ID,
                        type=RolesType.ROLES,
                    ),
                ],
            ),
        ),
    ),
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = UsersApi(api_client)
    response = api_instance.create_service_account(body=body)

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Dict, List, Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v1.model.slo_history_monitor import SLOHistoryMonitor
    from datadog_api_client.v1.model.slo_history_sli_data import SLOHistorySLIData
    from datadog_api_client.v1.model.slo_history_metrics import SLOHistoryMetrics
    from datadog_api_client.v1.model.slo_threshold import SLOThreshold
    from datadog_api_client.v1.model.slo_type import SLOType
    from datadog_api_client.v1.model.slo_type_numeric import SLOTypeNumeric


class SLOHistoryResponseData(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v1.model.slo_history_monitor import SLOHistoryMonitor
        from datadog_api_client.v1.model.slo_history_sli_data import SLOHistorySLIData
        from datadog_api_client.v1.model.slo_history_metrics import SLOHistoryMetrics
        from datadog_api_client.v1.model.slo_threshold import SLOThreshold
        from datadog_api_client.v1.model.slo_type import SLOType
        from datadog_api_client.v1.model.slo_type_numeric import SLOTypeNumeric

        return {
            "from_ts": (int,),
            "group_by": ([str],),
            "groups": ([SLOHistoryMonitor],),
            "monitors": ([SLOHistoryMonitor],),
            "overall": (SLOHistorySLIData,),
            "series": (SLOHistoryMetrics,),
            "thresholds": ({str: (SLOThreshold,)},),
            "to_ts": (int,),
            "type": (SLOType,),
            "type_id": (SLOTypeNumeric,),
        }

    attribute_map = {
        "from_ts": "from_ts",
        "group_by": "group_by",
        "groups": "groups",
        "monitors": "monitors",
        "overall": "overall",
        "series": "series",
        "thresholds": "thresholds",
        "to_ts": "to_ts",
        "type": "type",
        "type_id": "type_id",
    }

    def __init__(
        self_,
        from_ts: Union[int, UnsetType] = unset,
        group_by: Union[List[str], UnsetType] = unset,
        groups: Union[List[SLOHistoryMonitor], UnsetType] = unset,
        monitors: Union[List[SLOHistoryMonitor], UnsetType] = unset,
        overall: Union[SLOHistorySLIData, UnsetType] = unset,
        series: Union[SLOHistoryMetrics, UnsetType] = unset,
        thresholds: Union[Dict[str, SLOThreshold], UnsetType] = unset,
        to_ts: Union[int, UnsetType] = unset,
        type: Union[SLOType, UnsetType] = unset,
        type_id: Union[SLOTypeNumeric, UnsetType] = unset,
        **kwargs,
    ):
        """
        An array of service level objective objects.

        :param from_ts: The ``from`` timestamp in epoch seconds.
        :type from_ts: int, optional

        :param group_by: For ``metric`` based SLOs where the query includes a group-by clause, this represents the list of grouping parameters.

            This is not included in responses for ``monitor`` based SLOs.
        :type group_by: [str], optional

        :param groups: For grouped SLOs, this represents SLI data for specific groups.

            This is not included in the responses for ``metric`` based SLOs.
        :type groups: [SLOHistoryMonitor], optional

        :param monitors: For multi-monitor SLOs, this represents SLI data for specific monitors.

            This is not included in the responses for ``metric`` based SLOs.
        :type monitors: [SLOHistoryMonitor], optional

        :param overall: An object that holds an SLI value and its associated data. It can represent an SLO's overall SLI value.
            This can also represent the SLI value for a specific monitor in multi-monitor SLOs, or a group in grouped SLOs.
        :type overall: SLOHistorySLIData, optional

        :param series: A ``metric`` based SLO history response.

            This is not included in responses for ``monitor`` based SLOs.
        :type series: SLOHistoryMetrics, optional

        :param thresholds: mapping of string timeframe to the SLO threshold.
        :type thresholds: {str: (SLOThreshold,)}, optional

        :param to_ts: The ``to`` timestamp in epoch seconds.
        :type to_ts: int, optional

        :param type: The type of the service level objective.
        :type type: SLOType, optional

        :param type_id: A numeric representation of the type of the service level objective ( ``0`` for
            monitor, ``1`` for metric). Always included in service level objective responses.
            Ignored in create/update requests.
        :type type_id: SLOTypeNumeric, optional
        """
        if from_ts is not unset:
            kwargs["from_ts"] = from_ts
        if group_by is not unset:
            kwargs["group_by"] = group_by
        if groups is not unset:
            kwargs["groups"] = groups
        if monitors is not unset:
            kwargs["monitors"] = monitors
        if overall is not unset:
            kwargs["overall"] = overall
        if series is not unset:
            kwargs["series"] = series
        if thresholds is not unset:
            kwargs["thresholds"] = thresholds
        if to_ts is not unset:
            kwargs["to_ts"] = to_ts
        if type is not unset:
            kwargs["type"] = type
        if type_id is not unset:
            kwargs["type_id"] = type_id
        super().__init__(kwargs)



"""
Create a browser test returns "OK - Returns saved rumSettings." response
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.synthetics_api import SyntheticsApi
from datadog_api_client.v1.model.synthetics_browser_test import SyntheticsBrowserTest
from datadog_api_client.v1.model.synthetics_browser_test_config import SyntheticsBrowserTestConfig
from datadog_api_client.v1.model.synthetics_browser_test_rum_settings import SyntheticsBrowserTestRumSettings
from datadog_api_client.v1.model.synthetics_browser_test_type import SyntheticsBrowserTestType
from datadog_api_client.v1.model.synthetics_config_variable import SyntheticsConfigVariable
from datadog_api_client.v1.model.synthetics_config_variable_type import SyntheticsConfigVariableType
from datadog_api_client.v1.model.synthetics_device_id import SyntheticsDeviceID
from datadog_api_client.v1.model.synthetics_step import SyntheticsStep
from datadog_api_client.v1.model.synthetics_step_type import SyntheticsStepType
from datadog_api_client.v1.model.synthetics_test_ci_options import SyntheticsTestCiOptions
from datadog_api_client.v1.model.synthetics_test_execution_rule import SyntheticsTestExecutionRule
from datadog_api_client.v1.model.synthetics_test_options import SyntheticsTestOptions
from datadog_api_client.v1.model.synthetics_test_options_retry import SyntheticsTestOptionsRetry
from datadog_api_client.v1.model.synthetics_test_request import SyntheticsTestRequest

body = SyntheticsBrowserTest(
    config=SyntheticsBrowserTestConfig(
        assertions=[],
        config_variables=[
            SyntheticsConfigVariable(
                example="content-type",
                name="PROPERTY",
                pattern="content-type",
                type=SyntheticsConfigVariableType.TEXT,
            ),
        ],
        request=SyntheticsTestRequest(
            method="GET",
            url="https://datadoghq.com",
            certificate_domains=[
                "https://datadoghq.com",
            ],
        ),
        set_cookie="name:test",
    ),
    locations=[
        "aws:us-east-2",
    ],
    message="Test message",
    name="Example-Create_a_browser_test_returns_OK_Returns_saved_rumSettings_response",
    options=SyntheticsTestOptions(
        accept_self_signed=False,
        allow_insecure=True,
        device_ids=[
            SyntheticsDeviceID.TABLET,
        ],
        disable_cors=True,
        follow_redirects=True,
        min_failure_duration=10,
        min_location_failed=1,
        no_screenshot=True,
        retry=SyntheticsTestOptionsRetry(
            count=3,
            interval=10.0,
        ),
        rum_settings=SyntheticsBrowserTestRumSettings(
            is_enabled=True,
            application_id="mockApplicationId",
            client_token_id=12345,
        ),
        tick_every=300,
        ci=SyntheticsTestCiOptions(
            execution_rule=SyntheticsTestExecutionRule.SKIPPED,
        ),
        ignore_server_certificate_error=True,
        disable_csp=True,
        initial_navigation_timeout=200,
    ),
    tags=[
        "testing:browser",
    ],
    type=SyntheticsBrowserTestType.BROWSER,
    steps=[
        SyntheticsStep(
            allow_failure=False,
            is_critical=True,
            name="Refresh page",
            params=dict(),
            type=SyntheticsStepType.REFRESH,
        ),
    ],
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = SyntheticsApi(api_client)
    response = api_instance.create_synthetics_browser_test(body=body)

    print(response)



"""
Create a new dashboard with free_text widget
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.dashboards_api import DashboardsApi
from datadog_api_client.v1.model.dashboard import Dashboard
from datadog_api_client.v1.model.dashboard_layout_type import DashboardLayoutType
from datadog_api_client.v1.model.free_text_widget_definition import FreeTextWidgetDefinition
from datadog_api_client.v1.model.free_text_widget_definition_type import FreeTextWidgetDefinitionType
from datadog_api_client.v1.model.widget import Widget
from datadog_api_client.v1.model.widget_layout import WidgetLayout
from datadog_api_client.v1.model.widget_text_align import WidgetTextAlign

body = Dashboard(
    title="Example-Create_a_new_dashboard_with_free_text_widget",
    description=None,
    widgets=[
        Widget(
            layout=WidgetLayout(
                x=0,
                y=0,
                width=24,
                height=6,
            ),
            definition=FreeTextWidgetDefinition(
                type=FreeTextWidgetDefinitionType.FREE_TEXT,
                text="Example free text",
                color="#4d4d4d",
                font_size="auto",
                text_align=WidgetTextAlign.LEFT,
            ),
        ),
    ],
    template_variables=[],
    layout_type=DashboardLayoutType.FREE,
    is_read_only=False,
    notify_list=[],
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = DashboardsApi(api_client)
    response = api_instance.create_dashboard(body=body)

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    none_type,
    unset,
    UnsetType,
)


class HourlyUsageMeasurement(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "usage_type": (str,),
            "value": (int, none_type),
        }

    attribute_map = {
        "usage_type": "usage_type",
        "value": "value",
    }

    def __init__(
        self_, usage_type: Union[str, UnsetType] = unset, value: Union[int, none_type, UnsetType] = unset, **kwargs
    ):
        """
        Usage amount for a given usage type.

        :param usage_type: Type of usage.
        :type usage_type: str, optional

        :param value: Contains the number measured for the given usage_type during the hour.
        :type value: int, none_type, optional
        """
        if usage_type is not unset:
            kwargs["usage_type"] = usage_type
        if value is not unset:
            kwargs["value"] = value
        super().__init__(kwargs)



"""
Create an incident attachment returns "OK" response
"""

from os import environ
from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v2.api.incidents_api import IncidentsApi
from datadog_api_client.v2.model.incident_attachment_link_attachment_type import IncidentAttachmentLinkAttachmentType
from datadog_api_client.v2.model.incident_attachment_link_attributes import IncidentAttachmentLinkAttributes
from datadog_api_client.v2.model.incident_attachment_link_attributes_attachment_object import (
    IncidentAttachmentLinkAttributesAttachmentObject,
)
from datadog_api_client.v2.model.incident_attachment_type import IncidentAttachmentType
from datadog_api_client.v2.model.incident_attachment_update_data import IncidentAttachmentUpdateData
from datadog_api_client.v2.model.incident_attachment_update_request import IncidentAttachmentUpdateRequest

# there is a valid "incident" in the system
INCIDENT_DATA_ID = environ["INCIDENT_DATA_ID"]

body = IncidentAttachmentUpdateRequest(
    data=[
        IncidentAttachmentUpdateData(
            type=IncidentAttachmentType.INCIDENT_ATTACHMENTS,
            attributes=IncidentAttachmentLinkAttributes(
                attachment_type=IncidentAttachmentLinkAttachmentType.LINK,
                attachment=IncidentAttachmentLinkAttributesAttachmentObject(
                    document_url="https://www.example.com/doc",
                    title="Example-Create_an_incident_attachment_returns_OK_response",
                ),
            ),
        ),
    ],
)

configuration = Configuration()
configuration.unstable_operations["update_incident_attachments"] = True
with ApiClient(configuration) as api_client:
    api_instance = IncidentsApi(api_client)
    response = api_instance.update_incident_attachments(incident_id=INCIDENT_DATA_ID, body=body)

    print(response)



"""
Delete a Cloud Workload Security Agent rule returns "OK" response
"""

from os import environ
from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v2.api.cloud_workload_security_api import CloudWorkloadSecurityApi

# there is a valid "agent_rule" in the system
AGENT_RULE_DATA_ID = environ["AGENT_RULE_DATA_ID"]

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = CloudWorkloadSecurityApi(api_client)
    api_instance.delete_cloud_workload_security_agent_rule(
        agent_rule_id=AGENT_RULE_DATA_ID,
    )



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import List, Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v1.model.log_query_definition import LogQueryDefinition
    from datadog_api_client.v1.model.widget_display_type import WidgetDisplayType
    from datadog_api_client.v1.model.widget_formula import WidgetFormula
    from datadog_api_client.v1.model.timeseries_widget_expression_alias import TimeseriesWidgetExpressionAlias
    from datadog_api_client.v1.model.process_query_definition import ProcessQueryDefinition
    from datadog_api_client.v1.model.formula_and_function_query_definition import FormulaAndFunctionQueryDefinition
    from datadog_api_client.v1.model.formula_and_function_response_format import FormulaAndFunctionResponseFormat
    from datadog_api_client.v1.model.widget_request_style import WidgetRequestStyle
    from datadog_api_client.v1.model.formula_and_function_metric_query_definition import (
        FormulaAndFunctionMetricQueryDefinition,
    )
    from datadog_api_client.v1.model.formula_and_function_event_query_definition import (
        FormulaAndFunctionEventQueryDefinition,
    )
    from datadog_api_client.v1.model.formula_and_function_process_query_definition import (
        FormulaAndFunctionProcessQueryDefinition,
    )
    from datadog_api_client.v1.model.formula_and_function_apm_dependency_stats_query_definition import (
        FormulaAndFunctionApmDependencyStatsQueryDefinition,
    )
    from datadog_api_client.v1.model.formula_and_function_apm_resource_stats_query_definition import (
        FormulaAndFunctionApmResourceStatsQueryDefinition,
    )


class TimeseriesWidgetRequest(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v1.model.log_query_definition import LogQueryDefinition
        from datadog_api_client.v1.model.widget_display_type import WidgetDisplayType
        from datadog_api_client.v1.model.widget_formula import WidgetFormula
        from datadog_api_client.v1.model.timeseries_widget_expression_alias import TimeseriesWidgetExpressionAlias
        from datadog_api_client.v1.model.process_query_definition import ProcessQueryDefinition
        from datadog_api_client.v1.model.formula_and_function_query_definition import FormulaAndFunctionQueryDefinition
        from datadog_api_client.v1.model.formula_and_function_response_format import FormulaAndFunctionResponseFormat
        from datadog_api_client.v1.model.widget_request_style import WidgetRequestStyle

        return {
            "apm_query": (LogQueryDefinition,),
            "audit_query": (LogQueryDefinition,),
            "display_type": (WidgetDisplayType,),
            "event_query": (LogQueryDefinition,),
            "formulas": ([WidgetFormula],),
            "log_query": (LogQueryDefinition,),
            "metadata": ([TimeseriesWidgetExpressionAlias],),
            "network_query": (LogQueryDefinition,),
            "on_right_yaxis": (bool,),
            "process_query": (ProcessQueryDefinition,),
            "profile_metrics_query": (LogQueryDefinition,),
            "q": (str,),
            "queries": ([FormulaAndFunctionQueryDefinition],),
            "response_format": (FormulaAndFunctionResponseFormat,),
            "rum_query": (LogQueryDefinition,),
            "security_query": (LogQueryDefinition,),
            "style": (WidgetRequestStyle,),
        }

    attribute_map = {
        "apm_query": "apm_query",
        "audit_query": "audit_query",
        "display_type": "display_type",
        "event_query": "event_query",
        "formulas": "formulas",
        "log_query": "log_query",
        "metadata": "metadata",
        "network_query": "network_query",
        "on_right_yaxis": "on_right_yaxis",
        "process_query": "process_query",
        "profile_metrics_query": "profile_metrics_query",
        "q": "q",
        "queries": "queries",
        "response_format": "response_format",
        "rum_query": "rum_query",
        "security_query": "security_query",
        "style": "style",
    }

    def __init__(
        self_,
        apm_query: Union[LogQueryDefinition, UnsetType] = unset,
        audit_query: Union[LogQueryDefinition, UnsetType] = unset,
        display_type: Union[WidgetDisplayType, UnsetType] = unset,
        event_query: Union[LogQueryDefinition, UnsetType] = unset,
        formulas: Union[List[WidgetFormula], UnsetType] = unset,
        log_query: Union[LogQueryDefinition, UnsetType] = unset,
        metadata: Union[List[TimeseriesWidgetExpressionAlias], UnsetType] = unset,
        network_query: Union[LogQueryDefinition, UnsetType] = unset,
        on_right_yaxis: Union[bool, UnsetType] = unset,
        process_query: Union[ProcessQueryDefinition, UnsetType] = unset,
        profile_metrics_query: Union[LogQueryDefinition, UnsetType] = unset,
        q: Union[str, UnsetType] = unset,
        queries: Union[
            List[
                Union[
                    FormulaAndFunctionQueryDefinition,
                    FormulaAndFunctionMetricQueryDefinition,
                    FormulaAndFunctionEventQueryDefinition,
                    FormulaAndFunctionProcessQueryDefinition,
                    FormulaAndFunctionApmDependencyStatsQueryDefinition,
                    FormulaAndFunctionApmResourceStatsQueryDefinition,
                ]
            ],
            UnsetType,
        ] = unset,
        response_format: Union[FormulaAndFunctionResponseFormat, UnsetType] = unset,
        rum_query: Union[LogQueryDefinition, UnsetType] = unset,
        security_query: Union[LogQueryDefinition, UnsetType] = unset,
        style: Union[WidgetRequestStyle, UnsetType] = unset,
        **kwargs,
    ):
        """
        Updated timeseries widget.

        :param apm_query: The log query.
        :type apm_query: LogQueryDefinition, optional

        :param audit_query: The log query.
        :type audit_query: LogQueryDefinition, optional

        :param display_type: Type of display to use for the request.
        :type display_type: WidgetDisplayType, optional

        :param event_query: The log query.
        :type event_query: LogQueryDefinition, optional

        :param formulas: List of formulas that operate on queries.
        :type formulas: [WidgetFormula], optional

        :param log_query: The log query.
        :type log_query: LogQueryDefinition, optional

        :param metadata: Used to define expression aliases.
        :type metadata: [TimeseriesWidgetExpressionAlias], optional

        :param network_query: The log query.
        :type network_query: LogQueryDefinition, optional

        :param on_right_yaxis: Whether or not to display a second y-axis on the right.
        :type on_right_yaxis: bool, optional

        :param process_query: The process query to use in the widget.
        :type process_query: ProcessQueryDefinition, optional

        :param profile_metrics_query: The log query.
        :type profile_metrics_query: LogQueryDefinition, optional

        :param q: Widget query.
        :type q: str, optional

        :param queries: List of queries that can be returned directly or used in formulas.
        :type queries: [FormulaAndFunctionQueryDefinition], optional

        :param response_format: Timeseries or Scalar response.
        :type response_format: FormulaAndFunctionResponseFormat, optional

        :param rum_query: The log query.
        :type rum_query: LogQueryDefinition, optional

        :param security_query: The log query.
        :type security_query: LogQueryDefinition, optional

        :param style: Define request widget style.
        :type style: WidgetRequestStyle, optional
        """
        if apm_query is not unset:
            kwargs["apm_query"] = apm_query
        if audit_query is not unset:
            kwargs["audit_query"] = audit_query
        if display_type is not unset:
            kwargs["display_type"] = display_type
        if event_query is not unset:
            kwargs["event_query"] = event_query
        if formulas is not unset:
            kwargs["formulas"] = formulas
        if log_query is not unset:
            kwargs["log_query"] = log_query
        if metadata is not unset:
            kwargs["metadata"] = metadata
        if network_query is not unset:
            kwargs["network_query"] = network_query
        if on_right_yaxis is not unset:
            kwargs["on_right_yaxis"] = on_right_yaxis
        if process_query is not unset:
            kwargs["process_query"] = process_query
        if profile_metrics_query is not unset:
            kwargs["profile_metrics_query"] = profile_metrics_query
        if q is not unset:
            kwargs["q"] = q
        if queries is not unset:
            kwargs["queries"] = queries
        if response_format is not unset:
            kwargs["response_format"] = response_format
        if rum_query is not unset:
            kwargs["rum_query"] = rum_query
        if security_query is not unset:
            kwargs["security_query"] = security_query
        if style is not unset:
            kwargs["style"] = style
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import List, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
)


if TYPE_CHECKING:
    from datadog_api_client.v1.model.list_stream_column import ListStreamColumn
    from datadog_api_client.v1.model.list_stream_query import ListStreamQuery
    from datadog_api_client.v1.model.list_stream_response_format import ListStreamResponseFormat


class ListStreamWidgetRequest(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v1.model.list_stream_column import ListStreamColumn
        from datadog_api_client.v1.model.list_stream_query import ListStreamQuery
        from datadog_api_client.v1.model.list_stream_response_format import ListStreamResponseFormat

        return {
            "columns": ([ListStreamColumn],),
            "query": (ListStreamQuery,),
            "response_format": (ListStreamResponseFormat,),
        }

    attribute_map = {
        "columns": "columns",
        "query": "query",
        "response_format": "response_format",
    }

    def __init__(
        self_,
        columns: List[ListStreamColumn],
        query: ListStreamQuery,
        response_format: ListStreamResponseFormat,
        **kwargs,
    ):
        """
        Updated list stream widget.

        :param columns: Widget columns.
        :type columns: [ListStreamColumn]

        :param query: Updated list stream widget.
        :type query: ListStreamQuery

        :param response_format: Widget response format.
        :type response_format: ListStreamResponseFormat
        """
        super().__init__(kwargs)

        self_.columns = columns
        self_.query = query
        self_.response_format = response_format



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    datetime,
    unset,
    UnsetType,
)


class UsageBillableSummaryBody(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "account_billable_usage": (int,),
            "elapsed_usage_hours": (int,),
            "first_billable_usage_hour": (datetime,),
            "last_billable_usage_hour": (datetime,),
            "org_billable_usage": (int,),
            "percentage_in_account": (float,),
            "usage_unit": (str,),
        }

    attribute_map = {
        "account_billable_usage": "account_billable_usage",
        "elapsed_usage_hours": "elapsed_usage_hours",
        "first_billable_usage_hour": "first_billable_usage_hour",
        "last_billable_usage_hour": "last_billable_usage_hour",
        "org_billable_usage": "org_billable_usage",
        "percentage_in_account": "percentage_in_account",
        "usage_unit": "usage_unit",
    }

    def __init__(
        self_,
        account_billable_usage: Union[int, UnsetType] = unset,
        elapsed_usage_hours: Union[int, UnsetType] = unset,
        first_billable_usage_hour: Union[datetime, UnsetType] = unset,
        last_billable_usage_hour: Union[datetime, UnsetType] = unset,
        org_billable_usage: Union[int, UnsetType] = unset,
        percentage_in_account: Union[float, UnsetType] = unset,
        usage_unit: Union[str, UnsetType] = unset,
        **kwargs,
    ):
        """
        Response with properties for each aggregated usage type.

        :param account_billable_usage: The total account usage.
        :type account_billable_usage: int, optional

        :param elapsed_usage_hours: Elapsed usage hours for some billable product.
        :type elapsed_usage_hours: int, optional

        :param first_billable_usage_hour: The first billable hour for the org.
        :type first_billable_usage_hour: datetime, optional

        :param last_billable_usage_hour: The last billable hour for the org.
        :type last_billable_usage_hour: datetime, optional

        :param org_billable_usage: The number of units used within the billable timeframe.
        :type org_billable_usage: int, optional

        :param percentage_in_account: The percentage of account usage the org represents.
        :type percentage_in_account: float, optional

        :param usage_unit: Units pertaining to the usage.
        :type usage_unit: str, optional
        """
        if account_billable_usage is not unset:
            kwargs["account_billable_usage"] = account_billable_usage
        if elapsed_usage_hours is not unset:
            kwargs["elapsed_usage_hours"] = elapsed_usage_hours
        if first_billable_usage_hour is not unset:
            kwargs["first_billable_usage_hour"] = first_billable_usage_hour
        if last_billable_usage_hour is not unset:
            kwargs["last_billable_usage_hour"] = last_billable_usage_hour
        if org_billable_usage is not unset:
            kwargs["org_billable_usage"] = org_billable_usage
        if percentage_in_account is not unset:
            kwargs["percentage_in_account"] = percentage_in_account
        if usage_unit is not unset:
            kwargs["usage_unit"] = usage_unit
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v2.model.service_definition_v2_opsgenie import ServiceDefinitionV2Opsgenie


class ServiceDefinitionV2Integrations(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v2.model.service_definition_v2_opsgenie import ServiceDefinitionV2Opsgenie

        return {
            "opsgenie": (ServiceDefinitionV2Opsgenie,),
            "pagerduty": (str,),
        }

    attribute_map = {
        "opsgenie": "opsgenie",
        "pagerduty": "pagerduty",
    }

    def __init__(
        self_,
        opsgenie: Union[ServiceDefinitionV2Opsgenie, UnsetType] = unset,
        pagerduty: Union[str, UnsetType] = unset,
        **kwargs,
    ):
        """
        Third party integrations that Datadog supports.

        :param opsgenie: Opsgenie integration for the service.
        :type opsgenie: ServiceDefinitionV2Opsgenie, optional

        :param pagerduty: PagerDuty service URL for the service.
        :type pagerduty: str, optional
        """
        if opsgenie is not unset:
            kwargs["opsgenie"] = opsgenie
        if pagerduty is not unset:
            kwargs["pagerduty"] = pagerduty
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


class LogsQueryCompute(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "aggregation": (str,),
            "facet": (str,),
            "interval": (int,),
        }

    attribute_map = {
        "aggregation": "aggregation",
        "facet": "facet",
        "interval": "interval",
    }

    def __init__(
        self_, aggregation: str, facet: Union[str, UnsetType] = unset, interval: Union[int, UnsetType] = unset, **kwargs
    ):
        """
        Define computation for a log query.

        :param aggregation: The aggregation method.
        :type aggregation: str

        :param facet: Facet name.
        :type facet: str, optional

        :param interval: Define a time interval in seconds.
        :type interval: int, optional
        """
        if facet is not unset:
            kwargs["facet"] = facet
        if interval is not unset:
            kwargs["interval"] = interval
        super().__init__(kwargs)

        self_.aggregation = aggregation



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Any, Dict

from datadog_api_client.api_client import ApiClient, Endpoint as _Endpoint
from datadog_api_client.v1.model.logs_indexes_order import LogsIndexesOrder
from datadog_api_client.v1.model.logs_index_list_response import LogsIndexListResponse
from datadog_api_client.v1.model.logs_index import LogsIndex
from datadog_api_client.v1.model.logs_index_update_request import LogsIndexUpdateRequest


class LogsIndexesApi:
    """
    Manage configuration of `log indexes <https://docs.datadoghq.com/logs/indexes/>`_.
    You need an API and application key with Admin rights to interact with this endpoint.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        self._create_logs_index_endpoint = _Endpoint(
            settings={
                "response_type": (LogsIndex,),
                "auth": ["apiKeyAuth", "appKeyAuth"],
                "endpoint_path": "/api/v1/logs/config/indexes",
                "operation_id": "create_logs_index",
                "http_method": "POST",
                "version": "v1",
                "servers": None,
            },
            params_map={
                "body": {
                    "required": True,
                    "openapi_types": (LogsIndex,),
                    "location": "body",
                },
            },
            headers_map={"accept": ["application/json"], "content_type": ["application/json"]},
            api_client=api_client,
        )

        self._get_logs_index_endpoint = _Endpoint(
            settings={
                "response_type": (LogsIndex,),
                "auth": ["apiKeyAuth", "appKeyAuth"],
                "endpoint_path": "/api/v1/logs/config/indexes/{name}",
                "operation_id": "get_logs_index",
                "http_method": "GET",
                "version": "v1",
                "servers": None,
            },
            params_map={
                "name": {
                    "required": True,
                    "openapi_types": (str,),
                    "attribute": "name",
                    "location": "path",
                },
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )

        self._get_logs_index_order_endpoint = _Endpoint(
            settings={
                "response_type": (LogsIndexesOrder,),
                "auth": ["apiKeyAuth", "appKeyAuth", "AuthZ"],
                "endpoint_path": "/api/v1/logs/config/index-order",
                "operation_id": "get_logs_index_order",
                "http_method": "GET",
                "version": "v1",
                "servers": None,
            },
            params_map={},
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )

        self._list_log_indexes_endpoint = _Endpoint(
            settings={
                "response_type": (LogsIndexListResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth", "AuthZ"],
                "endpoint_path": "/api/v1/logs/config/indexes",
                "operation_id": "list_log_indexes",
                "http_method": "GET",
                "version": "v1",
                "servers": None,
            },
            params_map={},
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )

        self._update_logs_index_endpoint = _Endpoint(
            settings={
                "response_type": (LogsIndex,),
                "auth": ["apiKeyAuth", "appKeyAuth"],
                "endpoint_path": "/api/v1/logs/config/indexes/{name}",
                "operation_id": "update_logs_index",
                "http_method": "PUT",
                "version": "v1",
                "servers": None,
            },
            params_map={
                "name": {
                    "required": True,
                    "openapi_types": (str,),
                    "attribute": "name",
                    "location": "path",
                },
                "body": {
                    "required": True,
                    "openapi_types": (LogsIndexUpdateRequest,),
                    "location": "body",
                },
            },
            headers_map={"accept": ["application/json"], "content_type": ["application/json"]},
            api_client=api_client,
        )

        self._update_logs_index_order_endpoint = _Endpoint(
            settings={
                "response_type": (LogsIndexesOrder,),
                "auth": ["apiKeyAuth", "appKeyAuth"],
                "endpoint_path": "/api/v1/logs/config/index-order",
                "operation_id": "update_logs_index_order",
                "http_method": "PUT",
                "version": "v1",
                "servers": None,
            },
            params_map={
                "body": {
                    "required": True,
                    "openapi_types": (LogsIndexesOrder,),
                    "location": "body",
                },
            },
            headers_map={"accept": ["application/json"], "content_type": ["application/json"]},
            api_client=api_client,
        )

    def create_logs_index(
        self,
        body: LogsIndex,
    ) -> LogsIndex:
        """Create an index.

        Creates a new index. Returns the Index object passed in the request body when the request is successful.

        :param body: Object containing the new index.
        :type body: LogsIndex
        :rtype: LogsIndex
        """
        kwargs: Dict[str, Any] = {}
        kwargs["body"] = body

        return self._create_logs_index_endpoint.call_with_http_info(**kwargs)

    def get_logs_index(
        self,
        name: str,
    ) -> LogsIndex:
        """Get an index.

        Get one log index from your organization. This endpoint takes no JSON arguments.

        :param name: Name of the log index.
        :type name: str
        :rtype: LogsIndex
        """
        kwargs: Dict[str, Any] = {}
        kwargs["name"] = name

        return self._get_logs_index_endpoint.call_with_http_info(**kwargs)

    def get_logs_index_order(
        self,
    ) -> LogsIndexesOrder:
        """Get indexes order.

        Get the current order of your log indexes. This endpoint takes no JSON arguments.

        :rtype: LogsIndexesOrder
        """
        kwargs: Dict[str, Any] = {}
        return self._get_logs_index_order_endpoint.call_with_http_info(**kwargs)

    def list_log_indexes(
        self,
    ) -> LogsIndexListResponse:
        """Get all indexes.

        The Index object describes the configuration of a log index.
        This endpoint returns an array of the ``LogIndex`` objects of your organization.

        :rtype: LogsIndexListResponse
        """
        kwargs: Dict[str, Any] = {}
        return self._list_log_indexes_endpoint.call_with_http_info(**kwargs)

    def update_logs_index(
        self,
        name: str,
        body: LogsIndexUpdateRequest,
    ) -> LogsIndex:
        """Update an index.

        Update an index as identified by its name.
        Returns the Index object passed in the request body when the request is successful.

        Using the ``PUT`` method updates your index’s configuration by **replacing**
        your current configuration with the new one sent to your Datadog organization.

        :param name: Name of the log index.
        :type name: str
        :param body: Object containing the new ``LogsIndexUpdateRequest``.
        :type body: LogsIndexUpdateRequest
        :rtype: LogsIndex
        """
        kwargs: Dict[str, Any] = {}
        kwargs["name"] = name

        kwargs["body"] = body

        return self._update_logs_index_endpoint.call_with_http_info(**kwargs)

    def update_logs_index_order(
        self,
        body: LogsIndexesOrder,
    ) -> LogsIndexesOrder:
        """Update indexes order.

        This endpoint updates the index order of your organization.
        It returns the index order object passed in the request body when the request is successful.

        :param body: Object containing the new ordered list of index names
        :type body: LogsIndexesOrder
        :rtype: LogsIndexesOrder
        """
        kwargs: Dict[str, Any] = {}
        kwargs["body"] = body

        return self._update_logs_index_order_endpoint.call_with_http_info(**kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


class WidgetLayout(ModelNormal):
    validations = {
        "height": {
            "inclusive_minimum": 0,
        },
        "width": {
            "inclusive_minimum": 0,
        },
        "x": {
            "inclusive_minimum": 0,
        },
        "y": {
            "inclusive_minimum": 0,
        },
    }

    @cached_property
    def openapi_types(_):
        return {
            "height": (int,),
            "is_column_break": (bool,),
            "width": (int,),
            "x": (int,),
            "y": (int,),
        }

    attribute_map = {
        "height": "height",
        "is_column_break": "is_column_break",
        "width": "width",
        "x": "x",
        "y": "y",
    }

    def __init__(
        self_, height: int, width: int, x: int, y: int, is_column_break: Union[bool, UnsetType] = unset, **kwargs
    ):
        """
        The layout for a widget on a ``free`` or **new dashboard layout** dashboard.

        :param height: The height of the widget. Should be a non-negative integer.
        :type height: int

        :param is_column_break: Whether the widget should be the first one on the second column in high density or not.
            **Note** : Only for the **new dashboard layout** and only one widget in the dashboard should have this property set to ``true``.
        :type is_column_break: bool, optional

        :param width: The width of the widget. Should be a non-negative integer.
        :type width: int

        :param x: The position of the widget on the x (horizontal) axis. Should be a non-negative integer.
        :type x: int

        :param y: The position of the widget on the y (vertical) axis. Should be a non-negative integer.
        :type y: int
        """
        if is_column_break is not unset:
            kwargs["is_column_break"] = is_column_break
        super().__init__(kwargs)

        self_.height = height
        self_.width = width
        self_.x = x
        self_.y = y



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


class OrganizationSettingsSaml(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "enabled": (bool,),
        }

    attribute_map = {
        "enabled": "enabled",
    }

    def __init__(self_, enabled: Union[bool, UnsetType] = unset, **kwargs):
        """
        Set the boolean property enabled to enable or disable single sign on with SAML.
        See the SAML documentation for more information about all SAML settings.

        :param enabled: Whether or not SAML is enabled for this organization.
        :type enabled: bool, optional
        """
        if enabled is not unset:
            kwargs["enabled"] = enabled
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


class WidgetAxis(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "include_zero": (bool,),
            "label": (str,),
            "max": (str,),
            "min": (str,),
            "scale": (str,),
        }

    attribute_map = {
        "include_zero": "include_zero",
        "label": "label",
        "max": "max",
        "min": "min",
        "scale": "scale",
    }

    def __init__(
        self_,
        include_zero: Union[bool, UnsetType] = unset,
        label: Union[str, UnsetType] = unset,
        max: Union[str, UnsetType] = unset,
        min: Union[str, UnsetType] = unset,
        scale: Union[str, UnsetType] = unset,
        **kwargs,
    ):
        """
        Axis controls for the widget.

        :param include_zero: True includes zero.
        :type include_zero: bool, optional

        :param label: The label of the axis to display on the graph.
        :type label: str, optional

        :param max: Specifies the maximum value to show on the y-axis. It takes a number, or auto for default behavior.
        :type max: str, optional

        :param min: Specifies minimum value to show on the y-axis. It takes a number, or auto for default behavior.
        :type min: str, optional

        :param scale: Specifies the scale type. Possible values are ``linear`` , ``log`` , ``sqrt`` , ``pow##`` (for example ``pow2`` , ``pow0.5`` etc.).
        :type scale: str, optional
        """
        if include_zero is not unset:
            kwargs["include_zero"] = include_zero
        if label is not unset:
            kwargs["label"] = label
        if max is not unset:
            kwargs["max"] = max
        if min is not unset:
            kwargs["min"] = min
        if scale is not unset:
            kwargs["scale"] = scale
        super().__init__(kwargs)



"""
Create a new dashboard with iframe widget
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.dashboards_api import DashboardsApi
from datadog_api_client.v1.model.dashboard import Dashboard
from datadog_api_client.v1.model.dashboard_layout_type import DashboardLayoutType
from datadog_api_client.v1.model.i_frame_widget_definition import IFrameWidgetDefinition
from datadog_api_client.v1.model.i_frame_widget_definition_type import IFrameWidgetDefinitionType
from datadog_api_client.v1.model.widget import Widget
from datadog_api_client.v1.model.widget_layout import WidgetLayout

body = Dashboard(
    title="Example-Create_a_new_dashboard_with_iframe_widget",
    description="",
    widgets=[
        Widget(
            layout=WidgetLayout(
                x=0,
                y=0,
                width=12,
                height=12,
            ),
            definition=IFrameWidgetDefinition(
                type=IFrameWidgetDefinitionType.IFRAME,
                url="https://docs.datadoghq.com/api/latest/",
            ),
        ),
    ],
    template_variables=[],
    layout_type=DashboardLayoutType.FREE,
    is_read_only=False,
    notify_list=[],
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = DashboardsApi(api_client)
    response = api_instance.create_dashboard(body=body)

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


class SyntheticsTestRequestCertificateItem(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "content": (str,),
            "filename": (str,),
            "updated_at": (str,),
        }

    attribute_map = {
        "content": "content",
        "filename": "filename",
        "updated_at": "updatedAt",
    }

    def __init__(
        self_,
        content: Union[str, UnsetType] = unset,
        filename: Union[str, UnsetType] = unset,
        updated_at: Union[str, UnsetType] = unset,
        **kwargs,
    ):
        """
        Define a request certificate.

        :param content: Content of the certificate or key.
        :type content: str, optional

        :param filename: File name for the certificate or key.
        :type filename: str, optional

        :param updated_at: Date of update of the certificate or key, ISO format.
        :type updated_at: str, optional
        """
        if content is not unset:
            kwargs["content"] = content
        if filename is not unset:
            kwargs["filename"] = filename
        if updated_at is not unset:
            kwargs["updated_at"] = updated_at
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
)


if TYPE_CHECKING:
    from datadog_api_client.v2.model.cloud_configuration_compliance_rule_options import (
        CloudConfigurationComplianceRuleOptions,
    )


class CloudConfigurationRuleOptions(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v2.model.cloud_configuration_compliance_rule_options import (
            CloudConfigurationComplianceRuleOptions,
        )

        return {
            "compliance_rule_options": (CloudConfigurationComplianceRuleOptions,),
        }

    attribute_map = {
        "compliance_rule_options": "complianceRuleOptions",
    }

    def __init__(self_, compliance_rule_options: CloudConfigurationComplianceRuleOptions, **kwargs):
        """
        Options on cloud configuration rules.

        :param compliance_rule_options: Options for cloud_configuration rules.
            Fields ``resourceType`` and ``regoRule`` are mandatory when managing custom ``cloud_configuration`` rules.
        :type compliance_rule_options: CloudConfigurationComplianceRuleOptions
        """
        super().__init__(kwargs)

        self_.compliance_rule_options = compliance_rule_options



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Any, Dict, Union

from datadog_api_client.api_client import ApiClient, Endpoint as _Endpoint
from datadog_api_client.model_utils import (
    UnsetType,
    unset,
)
from datadog_api_client.v1.model.event_list_response import EventListResponse
from datadog_api_client.v1.model.event_priority import EventPriority
from datadog_api_client.v1.model.event_create_response import EventCreateResponse
from datadog_api_client.v1.model.event_create_request import EventCreateRequest
from datadog_api_client.v1.model.event_response import EventResponse


class EventsApi:
    """
    The events service allows you to programmatically post events to the event stream
    and fetch events from the event stream. Events are limited to 4000 characters.
    If an event is sent out with a message containing more than 4000 characters, only the
    first 4000 characters are displayed.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        self._create_event_endpoint = _Endpoint(
            settings={
                "response_type": (EventCreateResponse,),
                "auth": ["apiKeyAuth"],
                "endpoint_path": "/api/v1/events",
                "operation_id": "create_event",
                "http_method": "POST",
                "version": "v1",
                "servers": None,
            },
            params_map={
                "body": {
                    "required": True,
                    "openapi_types": (EventCreateRequest,),
                    "location": "body",
                },
            },
            headers_map={"accept": ["application/json"], "content_type": ["application/json"]},
            api_client=api_client,
        )

        self._get_event_endpoint = _Endpoint(
            settings={
                "response_type": (EventResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth", "AuthZ"],
                "endpoint_path": "/api/v1/events/{event_id}",
                "operation_id": "get_event",
                "http_method": "GET",
                "version": "v1",
                "servers": None,
            },
            params_map={
                "event_id": {
                    "required": True,
                    "openapi_types": (int,),
                    "attribute": "event_id",
                    "location": "path",
                },
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )

        self._list_events_endpoint = _Endpoint(
            settings={
                "response_type": (EventListResponse,),
                "auth": ["apiKeyAuth", "appKeyAuth", "AuthZ"],
                "endpoint_path": "/api/v1/events",
                "operation_id": "list_events",
                "http_method": "GET",
                "version": "v1",
                "servers": None,
            },
            params_map={
                "start": {
                    "required": True,
                    "openapi_types": (int,),
                    "attribute": "start",
                    "location": "query",
                },
                "end": {
                    "required": True,
                    "openapi_types": (int,),
                    "attribute": "end",
                    "location": "query",
                },
                "priority": {
                    "openapi_types": (EventPriority,),
                    "attribute": "priority",
                    "location": "query",
                },
                "sources": {
                    "openapi_types": (str,),
                    "attribute": "sources",
                    "location": "query",
                },
                "tags": {
                    "openapi_types": (str,),
                    "attribute": "tags",
                    "location": "query",
                },
                "unaggregated": {
                    "openapi_types": (bool,),
                    "attribute": "unaggregated",
                    "location": "query",
                },
                "exclude_aggregate": {
                    "openapi_types": (bool,),
                    "attribute": "exclude_aggregate",
                    "location": "query",
                },
                "page": {
                    "validation": {
                        "inclusive_maximum": 2147483647,
                    },
                    "openapi_types": (int,),
                    "attribute": "page",
                    "location": "query",
                },
            },
            headers_map={
                "accept": ["application/json"],
                "content_type": [],
            },
            api_client=api_client,
        )

    def create_event(
        self,
        body: EventCreateRequest,
    ) -> EventCreateResponse:
        """Post an event.

        This endpoint allows you to post events to the stream.
        Tag them, set priority and event aggregate them with other events.

        :param body: Event request object
        :type body: EventCreateRequest
        :rtype: EventCreateResponse
        """
        kwargs: Dict[str, Any] = {}
        kwargs["body"] = body

        return self._create_event_endpoint.call_with_http_info(**kwargs)

    def get_event(
        self,
        event_id: int,
    ) -> EventResponse:
        """Get an event.

        This endpoint allows you to query for event details.

        **Note** : If the event you’re querying contains markdown formatting of any kind,
        you may see characters such as ``%`` , ``\\`` , ``n`` in your output.

        :param event_id: The ID of the event.
        :type event_id: int
        :rtype: EventResponse
        """
        kwargs: Dict[str, Any] = {}
        kwargs["event_id"] = event_id

        return self._get_event_endpoint.call_with_http_info(**kwargs)

    def list_events(
        self,
        start: int,
        end: int,
        *,
        priority: Union[EventPriority, UnsetType] = unset,
        sources: Union[str, UnsetType] = unset,
        tags: Union[str, UnsetType] = unset,
        unaggregated: Union[bool, UnsetType] = unset,
        exclude_aggregate: Union[bool, UnsetType] = unset,
        page: Union[int, UnsetType] = unset,
    ) -> EventListResponse:
        """Get a list of events.

        The event stream can be queried and filtered by time, priority, sources and tags.

        **Notes** :

        *
          If the event you’re querying contains markdown formatting of any kind,
          you may see characters such as ``%`` , ``\\`` , ``n`` in your output.

        *
          This endpoint returns a maximum of ``1000`` most recent results. To return additional results,
          identify the last timestamp of the last result and set that as the ``end`` query time to
          paginate the results. You can also use the page parameter to specify which set of ``1000`` results to return.

        :param start: POSIX timestamp.
        :type start: int
        :param end: POSIX timestamp.
        :type end: int
        :param priority: Priority of your events, either ``low`` or ``normal``.
        :type priority: EventPriority, optional
        :param sources: A comma separated string of sources.
        :type sources: str, optional
        :param tags: A comma separated list indicating what tags, if any, should be used to filter the list of events.
        :type tags: str, optional
        :param unaggregated: Set unaggregated to ``true`` to return all events within the specified [ ``start`` , ``end`` ] timeframe.
            Otherwise if an event is aggregated to a parent event with a timestamp outside of the timeframe,
            it won't be available in the output. Aggregated events with ``is_aggregate=true`` in the response will still be returned unless exclude_aggregate is set to ``true.``
        :type unaggregated: bool, optional
        :param exclude_aggregate: Set ``exclude_aggregate`` to ``true`` to only return unaggregated events where ``is_aggregate=false`` in the response. If the ``exclude_aggregate`` parameter is set to ``true`` ,
            then the unaggregated parameter is ignored and will be ``true`` by default.
        :type exclude_aggregate: bool, optional
        :param page: By default 1000 results are returned per request. Set page to the number of the page to return with ``0`` being the first page. The page parameter can only be used
            when either unaggregated or exclude_aggregate is set to ``true.``
        :type page: int, optional
        :rtype: EventListResponse
        """
        kwargs: Dict[str, Any] = {}
        kwargs["start"] = start

        kwargs["end"] = end

        if priority is not unset:
            kwargs["priority"] = priority

        if sources is not unset:
            kwargs["sources"] = sources

        if tags is not unset:
            kwargs["tags"] = tags

        if unaggregated is not unset:
            kwargs["unaggregated"] = unaggregated

        if exclude_aggregate is not unset:
            kwargs["exclude_aggregate"] = exclude_aggregate

        if page is not unset:
            kwargs["page"] = page

        return self._list_events_endpoint.call_with_http_info(**kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
)


class ServiceLevelObjectiveQuery(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "denominator": (str,),
            "numerator": (str,),
        }

    attribute_map = {
        "denominator": "denominator",
        "numerator": "numerator",
    }

    def __init__(self_, denominator: str, numerator: str, **kwargs):
        """
        A metric-based SLO. **Required if type is metric**. Note that Datadog only allows the sum by aggregator
        to be used because this will sum up all request counts instead of averaging them, or taking the max or
        min of all of those requests.

        :param denominator: A Datadog metric query for total (valid) events.
        :type denominator: str

        :param numerator: A Datadog metric query for good events.
        :type numerator: str
        """
        super().__init__(kwargs)

        self_.denominator = denominator
        self_.numerator = numerator



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    none_type,
    unset,
    UnsetType,
)


class SLOHistoryMetricsSeriesMetadataUnit(ModelNormal):
    _nullable = True

    @cached_property
    def openapi_types(_):
        return {
            "family": (str,),
            "id": (int,),
            "name": (str,),
            "plural": (str, none_type),
            "scale_factor": (float,),
            "short_name": (str, none_type),
        }

    attribute_map = {
        "family": "family",
        "id": "id",
        "name": "name",
        "plural": "plural",
        "scale_factor": "scale_factor",
        "short_name": "short_name",
    }

    def __init__(
        self_,
        family: Union[str, UnsetType] = unset,
        id: Union[int, UnsetType] = unset,
        name: Union[str, UnsetType] = unset,
        plural: Union[str, none_type, UnsetType] = unset,
        scale_factor: Union[float, UnsetType] = unset,
        short_name: Union[str, none_type, UnsetType] = unset,
        **kwargs,
    ):
        """
        An Object of metric units.

        :param family: The family of metric unit, for example ``bytes`` is the family for ``kibibyte`` , ``byte`` , and ``bit`` units.
        :type family: str, optional

        :param id: The ID of the metric unit.
        :type id: int, optional

        :param name: The unit of the metric, for instance ``byte``.
        :type name: str, optional

        :param plural: The plural Unit of metric, for instance ``bytes``.
        :type plural: str, none_type, optional

        :param scale_factor: The scale factor of metric unit, for instance ``1.0``.
        :type scale_factor: float, optional

        :param short_name: A shorter and abbreviated version of the metric unit, for instance ``B``.
        :type short_name: str, none_type, optional
        """
        if family is not unset:
            kwargs["family"] = family
        if id is not unset:
            kwargs["id"] = id
        if name is not unset:
            kwargs["name"] = name
        if plural is not unset:
            kwargs["plural"] = plural
        if scale_factor is not unset:
            kwargs["scale_factor"] = scale_factor
        if short_name is not unset:
            kwargs["short_name"] = short_name
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    none_type,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v1.model.webhooks_integration_encoding import WebhooksIntegrationEncoding


class WebhooksIntegration(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v1.model.webhooks_integration_encoding import WebhooksIntegrationEncoding

        return {
            "custom_headers": (str, none_type),
            "encode_as": (WebhooksIntegrationEncoding,),
            "name": (str,),
            "payload": (str, none_type),
            "url": (str,),
        }

    attribute_map = {
        "custom_headers": "custom_headers",
        "encode_as": "encode_as",
        "name": "name",
        "payload": "payload",
        "url": "url",
    }

    def __init__(
        self_,
        name: str,
        url: str,
        custom_headers: Union[str, none_type, UnsetType] = unset,
        encode_as: Union[WebhooksIntegrationEncoding, UnsetType] = unset,
        payload: Union[str, none_type, UnsetType] = unset,
        **kwargs,
    ):
        """
        Datadog-Webhooks integration.

        :param custom_headers: If ``null`` , uses no header.
            If given a JSON payload, these will be headers attached to your webhook.
        :type custom_headers: str, none_type, optional

        :param encode_as: Encoding type. Can be given either ``json`` or ``form``.
        :type encode_as: WebhooksIntegrationEncoding, optional

        :param name: The name of the webhook. It corresponds with ``<WEBHOOK_NAME>``.
            Learn more on how to use it in
            `monitor notifications <https://docs.datadoghq.com/monitors/notify>`_.
        :type name: str

        :param payload: If ``null`` , uses the default payload.
            If given a JSON payload, the webhook returns the payload
            specified by the given payload.
            `Webhooks variable usage <https://docs.datadoghq.com/integrations/webhooks/#usage>`_.
        :type payload: str, none_type, optional

        :param url: URL of the webhook.
        :type url: str
        """
        if custom_headers is not unset:
            kwargs["custom_headers"] = custom_headers
        if encode_as is not unset:
            kwargs["encode_as"] = encode_as
        if payload is not unset:
            kwargs["payload"] = payload
        super().__init__(kwargs)

        self_.name = name
        self_.url = url



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelComposed,
    cached_property,
)


class DistributionWidgetHistogramRequestQuery(ModelComposed):
    def __init__(self, **kwargs):
        """
        Query definition for Distribution Widget Histogram Request

        :param aggregator: The aggregation methods available for metrics queries.
        :type aggregator: FormulaAndFunctionMetricAggregation, optional

        :param data_source: Data source for metrics queries.
        :type data_source: FormulaAndFunctionMetricDataSource

        :param name: Name of the query for use in formulas.
        :type name: str

        :param query: Metrics query definition.
        :type query: str

        :param compute: Compute options.
        :type compute: FormulaAndFunctionEventQueryDefinitionCompute

        :param group_by: Group by options.
        :type group_by: [FormulaAndFunctionEventQueryGroupBy], optional

        :param indexes: An array of index names to query in the stream. Omit or use `[]` to query all indexes at once.
        :type indexes: [str], optional

        :param search: Search options.
        :type search: FormulaAndFunctionEventQueryDefinitionSearch, optional

        :param storage: Option for storage location. Feature in Private Beta.
        :type storage: str, optional

        :param env: APM environment.
        :type env: str

        :param operation_name: Name of operation on service.
        :type operation_name: str, optional

        :param primary_tag_name: Name of the second primary tag used within APM. Required when `primary_tag_value` is specified. See https://docs.datadoghq.com/tracing/guide/setting_primary_tags_to_scope/#add-a-second-primary-tag-in-datadog
        :type primary_tag_name: str, optional

        :param primary_tag_value: Value of the second primary tag by which to filter APM data. `primary_tag_name` must also be specified.
        :type primary_tag_value: str, optional

        :param resource_name: APM resource name.
        :type resource_name: str, optional

        :param service: APM service name.
        :type service: str

        :param stat: APM resource stat name.
        :type stat: FormulaAndFunctionApmResourceStatName
        """
        super().__init__(kwargs)

    @cached_property
    def _composed_schemas(_):
        # we need this here to make our import statements work
        # we must store _composed_schemas in here so the code is only run
        # when we invoke this method. If we kept this at the class
        # level we would get an error because the class level
        # code would be run when this module is imported, and these composed
        # classes don't exist yet because their module has not finished
        # loading
        from datadog_api_client.v1.model.formula_and_function_metric_query_definition import (
            FormulaAndFunctionMetricQueryDefinition,
        )
        from datadog_api_client.v1.model.formula_and_function_event_query_definition import (
            FormulaAndFunctionEventQueryDefinition,
        )
        from datadog_api_client.v1.model.formula_and_function_apm_resource_stats_query_definition import (
            FormulaAndFunctionApmResourceStatsQueryDefinition,
        )

        return {
            "oneOf": [
                FormulaAndFunctionMetricQueryDefinition,
                FormulaAndFunctionEventQueryDefinition,
                FormulaAndFunctionApmResourceStatsQueryDefinition,
            ],
        }



"""
Create a new dashboard with funnel widget
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.dashboards_api import DashboardsApi
from datadog_api_client.v1.model.dashboard import Dashboard
from datadog_api_client.v1.model.dashboard_layout_type import DashboardLayoutType
from datadog_api_client.v1.model.funnel_query import FunnelQuery
from datadog_api_client.v1.model.funnel_request_type import FunnelRequestType
from datadog_api_client.v1.model.funnel_source import FunnelSource
from datadog_api_client.v1.model.funnel_widget_definition import FunnelWidgetDefinition
from datadog_api_client.v1.model.funnel_widget_definition_type import FunnelWidgetDefinitionType
from datadog_api_client.v1.model.funnel_widget_request import FunnelWidgetRequest
from datadog_api_client.v1.model.widget import Widget

body = Dashboard(
    layout_type=DashboardLayoutType.ORDERED,
    title="Example-Create_a_new_dashboard_with_funnel_widget with funnel widget",
    widgets=[
        Widget(
            definition=FunnelWidgetDefinition(
                type=FunnelWidgetDefinitionType.FUNNEL,
                requests=[
                    FunnelWidgetRequest(
                        query=FunnelQuery(
                            data_source=FunnelSource.RUM,
                            query_string="",
                            steps=[],
                        ),
                        request_type=FunnelRequestType.FUNNEL,
                    ),
                ],
            ),
        ),
    ],
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = DashboardsApi(api_client)
    response = api_instance.create_dashboard(body=body)

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    datetime,
    unset,
    UnsetType,
)


class RoleAttributes(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "created_at": (datetime,),
            "modified_at": (datetime,),
            "name": (str,),
            "user_count": (int,),
        }

    attribute_map = {
        "created_at": "created_at",
        "modified_at": "modified_at",
        "name": "name",
        "user_count": "user_count",
    }
    read_only_vars = {
        "created_at",
        "modified_at",
        "user_count",
    }

    def __init__(
        self_,
        created_at: Union[datetime, UnsetType] = unset,
        modified_at: Union[datetime, UnsetType] = unset,
        name: Union[str, UnsetType] = unset,
        user_count: Union[int, UnsetType] = unset,
        **kwargs,
    ):
        """
        Attributes of the role.

        :param created_at: Creation time of the role.
        :type created_at: datetime, optional

        :param modified_at: Time of last role modification.
        :type modified_at: datetime, optional

        :param name: The name of the role. The name is neither unique nor a stable identifier of the role.
        :type name: str, optional

        :param user_count: Number of users with that role.
        :type user_count: int, optional
        """
        if created_at is not unset:
            kwargs["created_at"] = created_at
        if modified_at is not unset:
            kwargs["modified_at"] = modified_at
        if name is not unset:
            kwargs["name"] = name
        if user_count is not unset:
            kwargs["user_count"] = user_count
        super().__init__(kwargs)



"""
Get a browser test returns "OK" response
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.synthetics_api import SyntheticsApi

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = SyntheticsApi(api_client)
    response = api_instance.get_browser_test(
        public_id="public_id",
    )

    print(response)



"""
Delete an AWS integration returns "OK" response
"""

from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v1.api.aws_integration_api import AWSIntegrationApi
from datadog_api_client.v1.model.aws_account_delete_request import AWSAccountDeleteRequest

body = AWSAccountDeleteRequest(
    account_id="1234567",
    role_name="DatadogAWSIntegrationRole",
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = AWSIntegrationApi(api_client)
    response = api_instance.delete_aws_account(body=body)

    print(response)



"""
Add custom timeboard dashboard to an existing dashboard list returns "OK" response
"""

from os import environ
from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v2.api.dashboard_lists_api import DashboardListsApi
from datadog_api_client.v2.model.dashboard_list_add_items_request import DashboardListAddItemsRequest
from datadog_api_client.v2.model.dashboard_list_item_request import DashboardListItemRequest
from datadog_api_client.v2.model.dashboard_type import DashboardType

# there is a valid "dashboard_list" in the system
DASHBOARD_LIST_ID = environ["DASHBOARD_LIST_ID"]

# there is a valid "dashboard" in the system
DASHBOARD_ID = environ["DASHBOARD_ID"]

body = DashboardListAddItemsRequest(
    dashboards=[
        DashboardListItemRequest(
            id=DASHBOARD_ID,
            type=DashboardType.CUSTOM_TIMEBOARD,
        ),
    ],
)

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = DashboardListsApi(api_client)
    response = api_instance.create_dashboard_list_items(dashboard_list_id=int(DASHBOARD_LIST_ID), body=body)

    print(response)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import Union, TYPE_CHECKING

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


if TYPE_CHECKING:
    from datadog_api_client.v1.model.synthetics_basic_auth_ntlm_type import SyntheticsBasicAuthNTLMType


class SyntheticsBasicAuthNTLM(ModelNormal):
    @cached_property
    def openapi_types(_):
        from datadog_api_client.v1.model.synthetics_basic_auth_ntlm_type import SyntheticsBasicAuthNTLMType

        return {
            "domain": (str,),
            "password": (str,),
            "type": (SyntheticsBasicAuthNTLMType,),
            "username": (str,),
            "workstation": (str,),
        }

    attribute_map = {
        "domain": "domain",
        "password": "password",
        "type": "type",
        "username": "username",
        "workstation": "workstation",
    }

    def __init__(
        self_,
        type: SyntheticsBasicAuthNTLMType,
        domain: Union[str, UnsetType] = unset,
        password: Union[str, UnsetType] = unset,
        username: Union[str, UnsetType] = unset,
        workstation: Union[str, UnsetType] = unset,
        **kwargs,
    ):
        """
        Object to handle ``NTLM`` authentication when performing the test.

        :param domain: Domain for the authentication to use when performing the test.
        :type domain: str, optional

        :param password: Password for the authentication to use when performing the test.
        :type password: str, optional

        :param type: The type of authentication to use when performing the test.
        :type type: SyntheticsBasicAuthNTLMType

        :param username: Username for the authentication to use when performing the test.
        :type username: str, optional

        :param workstation: Workstation for the authentication to use when performing the test.
        :type workstation: str, optional
        """
        if domain is not unset:
            kwargs["domain"] = domain
        if password is not unset:
            kwargs["password"] = password
        if username is not unset:
            kwargs["username"] = username
        if workstation is not unset:
            kwargs["workstation"] = workstation
        super().__init__(kwargs)

        self_.type = type



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations

from typing import List, Union

from datadog_api_client.model_utils import (
    ModelNormal,
    cached_property,
    unset,
    UnsetType,
)


class CloudConfigurationRuleComplianceSignalOptions(ModelNormal):
    @cached_property
    def openapi_types(_):
        return {
            "user_activation_status": (bool,),
            "user_group_by_fields": ([str],),
        }

    attribute_map = {
        "user_activation_status": "userActivationStatus",
        "user_group_by_fields": "userGroupByFields",
    }

    def __init__(
        self_,
        user_activation_status: Union[bool, UnsetType] = unset,
        user_group_by_fields: Union[List[str], UnsetType] = unset,
        **kwargs,
    ):
        """
        How to generate compliance signals. Useful for cloud_configuration rules only.

        :param user_activation_status: Whether signals will be sent.
        :type user_activation_status: bool, optional

        :param user_group_by_fields: Fields to use to group findings by when sending signals.
        :type user_group_by_fields: [str], optional
        """
        if user_activation_status is not unset:
            kwargs["user_activation_status"] = user_activation_status
        if user_group_by_fields is not unset:
            kwargs["user_group_by_fields"] = user_group_by_fields
        super().__init__(kwargs)



# Unless explicitly stated otherwise all files in this repository are licensed under the Apache-2.0 License.
# This product includes software developed at Datadog (https://www.datadoghq.com/).
# Copyright 2019-Present Datadog, Inc.
from __future__ import annotations


from datadog_api_client.model_utils import (
    ModelSimple,
    cached_property,
)

from typing import ClassVar


class WidgetViewMode(ModelSimple):
    """
    Define how you want the SLO to be displayed.

    :param value: Must be one of ["overall", "component", "both"].
    :type value: str
    """

    allowed_values = {
        "overall",
        "component",
        "both",
    }
    OVERALL: ClassVar["WidgetViewMode"]
    COMPONENT: ClassVar["WidgetViewMode"]
    BOTH: ClassVar["WidgetViewMode"]

    @cached_property
    def openapi_types(_):
        return {
            "value": (str,),
        }


WidgetViewMode.OVERALL = WidgetViewMode("overall")
WidgetViewMode.COMPONENT = WidgetViewMode("component")
WidgetViewMode.BOTH = WidgetViewMode("both")



"""
Get a user permissions returns "OK" response
"""

from os import environ
from datadog_api_client import ApiClient, Configuration
from datadog_api_client.v2.api.users_api import UsersApi

# there is a valid "user" in the system
USER_DATA_ID = environ["USER_DATA_ID"]

configuration = Configuration()
with ApiClient(configuration) as api_client:
    api_instance = UsersApi(api_client)
    response = api_instance.list_user_permissions(
        user_id=USER_DATA_ID,
    )

    print(response)
